<?php

defined( 'ABSPATH' ) || exit;

class Boostr_Shipping_Admin {
	public function __construct() {
		$this->init();

		if ( ! class_exists( 'Boostr_Shipping_Admin_GLS' ) ) {
			include_once 'boostr_shipping_admin_gls.php';
			$oBoostr_Shipping_Admin_GLS = new Boostr_Shipping_Admin_GLS();
		}

		if ( ! class_exists( 'Boostr_Shipping_Settings' ) ) {
			require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/boostr_shipping_settings.php';
			$boostr_shipping_settings = new Boostr_Shipping_Settings();
			$boostr_shipping_settings->register_actions();
		}
	}

	public function init() {
		$this->addActions();
		$this->addFilters();
	}

	public function addActions() {
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_action( 'admin_print_scripts-woocommerce_page_wc-settings', array( $this, 'enqueue_shipping_method_settings_scripts' ) );
		add_action( 'woocommerce_update_options', array( $this, 'update_range_prices' ) );
		add_action( 'wp_ajax_boostr_shipping_get_price_ranges', array( $this, 'get_price_ranges_callback' ) );
		add_action( 'init', array( $this, 'periodic_check_uploads_files' ) );
		add_action( 'admin_notices', array( $this, 'no_google_api_key_admin_notice' ) );
		add_action( 'admin_notices', array( $this, 'no_key_admin_notice' ) );
		add_action( 'init', array( $this, 'load_textdomain' ) );
	}

	public function addFilters() {
	}

	public function load_textdomain() {
		load_plugin_textdomain( 'woocommerce-boostr-shipping', false, BOOSTR_SHIPPING_PLUGIN_BASENAME_DIRNAME . '/languages' );
	}

	public function enqueue_scripts() {
		$screen = get_current_screen();

		if ( $screen->id === 'shop_order' ) {
			wp_enqueue_script( 'boostr-shipping-orders', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/js/admin/boostr-shipping-orders.js', array( 'jquery', 'jquery-ui-datepicker' ), BOOSTR_SHIPPING_VERSION, true );

			wp_localize_script(
				'boostr-shipping-orders',
				'boostr_shipping_orders_params',
				array(
					'ajax_url' => admin_url( 'admin-ajax.php' ),
					'nonce'    => wp_create_nonce( 'boostr-shipping-order-action' ),
					'strings'  => array(
						'an_error_occured'     => __( 'An error occured', 'woocommerce-boostr-shipping' ),
						'send_tt_mail_again'   => __( 'Send the Track & Trace mail again', 'woocommerce-boostr-shipping' ),
						'please_select_a_date' => __( 'Please select a date first', 'woocommerce-boostr-shipping' ),
					),
				)
			);
		}
	}

	public function enqueue_shipping_method_settings_scripts( $hook ) {
		$aAdminParams = array(
			'sMaximumTranslation'                 => __( 'Maximum cart total', 'woocommerce-boostr-shipping' ),
			'ajax_url'                            => admin_url( 'admin-ajax.php' ),
			'sBtnAddNewPriceRangeRowTranslation'  => __( 'Add row', 'woocommerce-boostr-shipping' ),
			'sCartTotalTranslation'               => __( 'Cart Total', 'woocommerce-boostr-shipping' ),
			'sCurrencySymbol'                     => get_woocommerce_currency_symbol(),
			'sWeightUnit'                         => get_option( 'woocommerce_weight_unit' ),

			'sShippingPriceTranslation'           => __( 'Shipping Price', 'woocommerce-boostr-shipping' ),
			'sWeightTranslation'                  => __( 'Weight', 'woocommerce-boostr-shipping' ),
			'sPriceTranslation'                   => __( 'Price', 'woocommerce-boostr-shipping' ),

			'sQuantityTranslation'                => __( 'Quantity', 'woocommerce-boostr-shipping' ),
			'sTitleTranslation'                   => __( 'Plugins.dk GLS shipping for WooCommerce', 'woocommerce-boostr-shipping' ),
			'sMinimumTranslation'                 => __( 'Minimum cart total', 'woocommerce-boostr-shipping' ),

			'sShippingRangeHelperTextTranslation' => __( 'In the price table below, you can choose to setup diffrent shipping prices, that will be based on the carts total of your choosen type. <br/> If the cart total falls outside of any of the choosen ranges, the shipping price will default to the highest shipping price  <br/> Please make sure to follow the woocommerce standart, and use a peroid as a decimal seperator.', 'woocommerce-boostr-shipping' ),
		);

		wp_enqueue_script( 'boostr-shipping-method-settings.js', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/js/admin/boostr-shipping-method-settings.js', array( 'jquery' ), BOOSTR_SHIPPING_VERSION );
		wp_localize_script( 'boostr-shipping-method-settings.js', 'BoostrShippingAdminParams', $aAdminParams );
		wp_register_style( 'admin_post_style', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/css/admin/boostr-shipping-admin.css', array(), BOOSTR_SHIPPING_VERSION );
		wp_enqueue_style( 'admin_post_style' );
	}

	public function no_google_api_key_admin_notice() {
		if ( ! empty( $_GET['page'] ) ) {
			//if on google map settings page, dont show notice
			if ( $_GET['page'] == 'boostr-shipping' ) {
				return '';
			}
		}

		$iGoogle_map_api = get_option( 'boostr_shipping_google_settings' )['google_map_api_key'];
		if ( empty( $iGoogle_map_api ) || strlen( $iGoogle_map_api ) < 5 ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p>
					<?php $boostr_shipping_admin_url = '<a href="' . admin_url() . '/admin.php?page=boostr-shipping&tab=boostr_shipping_google">' . __( 'Google Maps Settings', 'woocommerce-boostr-shipping' ) . '</a>'; ?>
					<?php printf( esc_html__( 'Please go to  %s and add a Google Maps API key, before using the plugins.dk GLS plugin.', 'woocommerce-boostr-shipping' ), wp_kses_post( $boostr_shipping_admin_url ) ); ?>
				</p>
			</div>
			<?php
		}
	}

	public function no_key_admin_notice() {
		if ( ! empty( $_GET['page'] ) && ! empty( $_GET['tab'] ) ) {
			//if on license settings page, dont show notice
			if ( $_GET['page'] == 'boostr-shipping' && $_GET['tab'] == 'boostr_shipping_general' ) {
				return '';
			}
		}

		$license_key = get_option( 'boostr_shipping_license_key' );

		if ( empty( $license_key ) ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p>
				<?php $sBoostr_shipping_settings_URL = '<a href="' . admin_url() . '/admin.php?page=boostr-shipping&tab=boostr_shipping_general">' . __( 'Plugins.dk GLS Settings', 'woocommerce-boostr-shipping' ) . '</a>'; ?>
				<?php printf( esc_html__( 'Your license seems to be inactive, please go to %s and activate your License Key', 'woocommerce-boostr-shipping' ), wp_kses_post( $sBoostr_shipping_settings_URL ) ); ?>
				</p>
			</div>
			<?php
		}
	}

	public function update_range_prices() {
		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			return;
		}

		$aPostKeys     = array_keys( $_POST );
		$oShippingData = json_decode( stripslashes_deep( $_POST[ $aPostKeys[0] ] ) );
		if ( isset( $oShippingData->iInstance_id ) ) {
			$iInstance_id      = $oShippingData->iInstance_id;
			$sRangeType        = $oShippingData->sRangeType;
			$oShippingRangeRow = json_decode( $oShippingData->oShippingRows )->oRows;
			update_option( $sRangeType . '_' . $iInstance_id, $oShippingRangeRow );

		}
	}

	public static function get_iso_3166( $iso = '' ) {
		if ( strlen( $iso ) > 2 || strlen( $iso ) < 1 ) {
			return array( 'error', 'Error retrieving ISO code' );
		}

		$filename  = 'countries.csv';
		$file      = fopen( dirname( __FILE__ ) . '/' . $filename, 'r' );
		$countries = array();

		while ( ( $line = fgetcsv( $file ) ) !== false ) {
			list( $iso2, $iso3, $name, $official_name, $numcode ) = $line;

			$countries[ $iso2 ] = array( $iso3, $name, $official_name, $numcode );
		}

		if ( array_key_exists( $iso, $countries ) ) {
			return $countries[ $iso ][3];
		} else {
			return array( 'error', 'ISO code not found in file!' );
		}
	}

	public static function get_interline_country_code( $country ) {
		$countries = array(
			'FR' => '001',
			'BE' => '002',
			'NL' => '003',
			'DE' => '004',
			'IT' => '005',
			'GB' => '006',
			'IE' => '007',
			'DK' => '008',
			'GR' => '009',
			'PT' => '010',
			'ES' => '011',
			'LU' => '019',
			'NO' => '028',
			'SE' => '030',
			'FI' => '032',
			'AT' => '038',
			'CH' => '039',
			'AD' => '043',
			'VA' => '045',
			'MT' => '046',
			'TR' => '052',
			'PL' => '060',
			'CZ' => '061',
			'SK' => '063',
			'HU' => '064',
			'RO' => '066',
			'AL' => '070',
			'UA' => '072',
			'BY' => '073',
			'MD' => '074',
			'RU' => '075',
			'GE' => '076',
			'AM' => '077',
			'AZ' => '078',
			'KZ' => '079',
			'TM' => '080',
			'UZ' => '081',
			'TJ' => '082',
			'KG' => '083',
			'SI' => '091',
			'HR' => '092',
			'BA' => '093',
			'MK' => '096',
			'ME' => '097',
			'RS' => '098',
			'BG' => '100',
			'CY' => '196',
			'MA' => '204',
			'DZ' => '208',
			'TN' => '212',
			'LY' => '216',
			'EG' => '220',
			'SD' => '224',
			'MR' => '228',
			'ML' => '232',
			'EE' => '233',
			'FO' => '234',
			'BF' => '236',
			'NE' => '240',
			'TD' => '244',
			'CV' => '247',
			'SN' => '248',
			'GM' => '252',
			'GW' => '257',
			'GN' => '260',
			'SL' => '264',
			'LR' => '268',
			'CI' => '272',
			'GH' => '276',
			'TG' => '280',
			'BJ' => '284',
			'NG' => '288',
			'GI' => '292',
			'CM' => '302',
			'GL' => '304',
			'CF' => '306',
			'GQ' => '310',
			'GA' => '314',
			'YE' => '653',
			'PK' => '662',
			'IN' => '664',
			'BD' => '666',
			'MV' => '667',
			'LK' => '669',
			'NP' => '672',
			'SM' => '674',
			'BT' => '675',
			'MM' => '676',
			'TH' => '680',
			'LA' => '684',
			'VN' => '690',
			'KH' => '696',
			'ID' => '700',
			'MY' => '701',
			'BN' => '703',
			'SG' => '706',
		);

		if ( isset( $countries[ $country ] ) ) {
			return $countries[ $country ];
		} else {
			return '';
		}
	}

	public static function check_order_shipping_method( $oOrder ) {
		if ( $oOrder instanceof WC_Order ) {
			$aShipping_methods = array( 'boostr_shipping_gls_pickup', 'boostr_shipping_gls_private', 'boostr_shipping_gls_business' );
			foreach ( $oOrder->get_shipping_methods() as $oShipping_method ) {
				//stripping the method for digits, since 2.6 added that to the method choosen
				$sShipping_method_stripped = preg_replace( '/\d/', '', $oShipping_method['method_id'] );
				if ( in_array( $sShipping_method_stripped, $aShipping_methods ) ) {
					return $sShipping_method_stripped;
				}
			}
		}
		return false;
	}

	public function get_price_ranges_callback() {
		if ( ! isset( $_POST['iInstance_id'], $_POST['sRangeType'] ) || ! current_user_can( 'manage_woocommerce' ) ) {
			wp_send_json_error();
		}

		$iInstance_id = $_POST['iInstance_id'];
		$sRangeType   = $_POST['sRangeType'];
		$response     = get_option( $sRangeType . '_' . $iInstance_id );

		wp_send_json_success( $response );
	}

	public static function get_order_weight( $order_id ) {
		$the_order = new WC_Order( $order_id );
		$weight    = 0;
		if ( sizeof( $the_order->get_items() ) > 0 ) {
			foreach ( $the_order->get_items() as $item ) {
				if ( $item['product_id'] > 0 ) {
					$_product = $item->get_product();
					if ( ! $_product->is_virtual() ) {
						$weight += floatval( $_product->get_weight() ) * (int) $item['qty'];
					}
				}
			}
		}
		return $weight;
	}

	/**
	 * Gets the labels upload directory path.
	 *
	 * @since 2.3.0
	 */
	public static function get_uploads_dir() {
		$upload            = wp_upload_dir();
		$upload_dir        = $upload['basedir'];
		$pluginsdk_gls_dir = $upload_dir . '/gls-labels';

		return $pluginsdk_gls_dir;
	}

	/**
	 * Creates the labels uploads folder.
	 *
	 * @since 2.3.0
	 */
	public static function create_uploads_dir() {
		$pluginsdk_gls_dir = self::get_uploads_dir();
		$result            = wp_mkdir_p( $pluginsdk_gls_dir );

		return $result;
	}

	/**
	 * Creates the labels uploads folder .htaccess and index.html files.
	 *
	 * @since 2.3.0
	 */
	public static function create_uploads_files() {
		$pluginsdk_gls_dir = self::get_uploads_dir();

		if ( ! self::create_uploads_dir() ) {
			return false;
		}

		$htaccess_content  = "Options -Indexes\n";
		$htaccess_content .= "Deny from all\n";

		$index_content = '';

		$htaccess_file_path = $pluginsdk_gls_dir . '/.htaccess';
		$index_file_path    = $pluginsdk_gls_dir . '/index.html';

		if ( ! file_exists( $htaccess_file_path ) || @file_get_contents( $htaccess_file_path ) !== $htaccess_content ) {
			@file_put_contents( $htaccess_file_path, $htaccess_content );
		}

		if ( ! file_exists( $index_file_path ) || @file_get_contents( $index_file_path ) !== $index_content ) {
			@file_put_contents( $index_file_path, $index_content );
		}

		// TODO: Check if files are created successfully.
		return true;
	}

	/**
	 * Periodically checks if the uploads protection files exist, and attempts to create them if they don't.
	 *
	 * @since 2.3.0
	 */
	public function periodic_check_uploads_files() {
		$check_uploads_files = get_transient( 'pluginsdk_gls_check_uploads_files' );

		if ( false === $check_uploads_files ) {
			self::create_uploads_files();
			set_transient( 'pluginsdk_gls_check_uploads_files', true, 12 * 3600 );
		}
	}
}
