<?php

defined( 'ABSPATH' ) || exit;

class Boostr_Shipping_Admin_GLS {
	public function __construct() {
		$this->init();
	}

	public function init() {
		$this->addActions();
		$this->addFilters();
	}

	public function addActions() {
		add_action( 'init', array( $this, 'shipping_methods_init' ) );

		add_filter( 'woocommerce_admin_order_actions', array( $this, 'add_order_actions' ), 100, 2 );
		add_action( 'admin_init', array( $this, 'export_csv_file_action' ) );
		add_action( 'admin_init', array( $this, 'export_csv_ftp_action' ) );

		add_filter( 'bulk_actions-edit-shop_order', array( $this, 'add_bulk_actions' ), 99, 1 );
		add_filter( 'handle_bulk_actions-edit-shop_order', array( $this, 'export_bulk_actions' ), 10, 3 );
		add_action( 'admin_notices', array( $this, 'export_bulk_admin_notices' ) );

		//meta box's
		add_action( 'add_meta_boxes', array( $this, 'boostr_shipping_add_meta_box_gls_label' ) );
		add_action( 'add_meta_boxes', array( $this, 'boostr_shipping_add_meta_box_gls_tracktrace' ) );

		//onClick's
		add_action( 'admin_init', array( $this, 'download_label' ) );

		//AJAX call's
		add_action( 'wp_ajax_gls_get_label', array( $this, 'gls_get_label_callback' ) );
		add_action( 'wp_ajax_gls_send_tracktrace', array( $this, 'boostr_shipping_gls_send_tracktrace_callback' ) );
		add_action( 'wp_ajax_custom_tracktrace_enable_own_tt', array( $this, 'boostr_shipping_custom_tracktrace_enable_own_tt_callback' ) );

		//Admin track trace column in order overview
		add_action( 'manage_shop_order_posts_custom_column', array( $this, 'admin_add_order_overview_tracktrace_column' ), 2 );
	}

	public function addFilters() {
		//Admin track trace column in order overview
		add_filter( 'manage_edit-shop_order_columns', array( $this, 'admin_order_tracktrace_column' ), 1 );
	}

	/* Loads the shipping methods */
	public function shipping_methods_init() {
		$license_key     = get_option( 'boostr_shipping_license_key' );
		$iGoogle_map_api = get_option( 'boostr_shipping_google_settings' );
		$iGoogle_map_api = $iGoogle_map_api['google_map_api_key'];

		if ( ! empty( $license_key ) ) {
			if ( ! empty( $iGoogle_map_api ) && strlen( $iGoogle_map_api ) > 5 ) {
				if ( ! class_exists( 'Boostr_Shipping_GLS_Pickup' ) ) {
					require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/methods/boostr_shipping_gls_pickup.php';
				}
			}

			if ( ! class_exists( 'Boostr_Shipping_GLS_Private' ) ) {
				require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/methods/boostr_shipping_gls_private.php';
			}

			if ( ! class_exists( 'Boostr_Shipping_GLS_Business' ) ) {
				require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/methods/boostr_shipping_gls_business.php';
			}
		}
	}

	public function boostr_shipping_add_meta_box_gls_tracktrace() {
		$screens = array( 'shop_order' );
		foreach ( $screens as $screen ) {
			add_meta_box(
				'gls_send_tracktrace',
				__( 'Track & Trace', 'woocommerce-boostr-shipping' ),
				array( $this, 'boostr_shipping_gls_tracktrace_admin_html_callback' ),
				$screen,
				'side'
			);
		}
	}

	public function boostr_shipping_gls_tracktrace_admin_html_callback( $post ) {
		$order = new WC_Order( $post->ID );
		if ( Boostr_Shipping_Admin::check_order_shipping_method( $order ) ) {
			?>

			<?php
			$options = get_option( 'boostr_shipping_gls_settings' );
			if ( ! empty( $options['custom_tracktrace_enable_own_tt'] ) && $options['custom_tracktrace_enable_own_tt'] == 'Yes' ) {
				?>
			<table>
				<tr>
					<td>
						<input type="text" id="custom_gls_tracktrace" size="11" value="<?php print ( ! empty( get_post_meta( $post->ID, 'tracktrace_id', true ) ) ? esc_attr( get_post_meta( $post->ID, 'tracktrace_id', true ) ) : '' ); ?>">
						<input type="button" id="custom_tracktrace_enable_own_tt" class="button button-primary" value="<?php print esc_attr__( 'Update', 'woocommerce-boostr-shipping' ); ?>">
					</td>
					<td>
					</td>
				</tr>
				<tr>
					<td colspan="2">
						<small>
							<?php print esc_html__( 'If you have your own track and trace ID, wich is generated elsewhere (eg. GLS program Interline) you can put it here', 'woocommerce-boostr-shipping' ); ?>
						</small>
					</td>
				</tr>
			</table>


			<!--Button that controls the GLS track and trace - will check if a track and trace already have been send - and see their is a label generated for it already -->
			<input type="button" id="send_gls_tracktrace" class="button button-primary"
				style="
				<?php
				if ( empty( get_post_meta( $post->ID )['tracktrace_id'][0] ) ) {
					echo 'display:none;';}
				?>
					"
				Value="
					<?php
					if ( ! empty( get_post_meta( $post->ID )['tracktrace_mail_last_attempt'][0] ) && isset( get_post_meta( $post->ID )['tracktrace_mail_last_attempt'][0] ) ) {
						$track_last_mail_attempt = get_post_meta( $post->ID )['tracktrace_mail_last_attempt'][0];
						if ( $track_last_mail_attempt == 'true' ) {
							//echo $track_last_mail_attempt;
							echo esc_attr__( 'Send email again', 'woocommerce-boostr-shipping' );
						} else {
							echo esc_attr__( 'Send email', 'woocommerce-boostr-shipping' );
						}
					} else {
						echo esc_attr__( 'Send email', 'woocommerce-boostr-shipping' );
					}

					?>
				">
				<?php
			}
			?>
			<table class="track_and_trace_table">
				<th>
					<tr>
						<td>
							Date
						</td>
						<td>
							Time
						</td>
						<td>
							ID
						</td>
					</tr>
				</th>
				<?php
				if ( ! empty( get_post_meta( $post->ID )['tracktrace_history_json'][0] ) && isset( get_post_meta( $post->ID )['tracktrace_history_json'][0] ) ) {
					$tracktrace_history = json_decode( get_post_meta( $post->ID )['tracktrace_history_json'][0] );
					foreach ( $tracktrace_history as $instance ) {

						?>
						<tr>
							<?php
							foreach ( $instance as $value ) {
								?>
								<td>
									<?php echo esc_html( $value ); ?>
								</td>
								<?php
							}
							?>
						</tr>
						<?php
					}
				} else {

				}
				?>
			</table>
			<?php
		}
	}

	public function boostr_shipping_custom_tracktrace_enable_own_tt_callback( $calledFromLabelMethod = false ) {
		// Only check nonce if called with AJAX.
		if ( $calledFromLabelMethod !== true ) {
			check_ajax_referer( 'boostr-shipping-order-action' );
		}

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		if ( filter_var( $_POST['post_id'], FILTER_VALIDATE_INT ) === false ) {
			return;
		}

		$post_ID            = absint( $_POST['post_id'] );
		$track_and_trace_ID = $_POST['track_and_trace_ID'];

		if ( $post_ID > 0 && ! empty( $track_and_trace_ID ) ) {

			update_post_meta( $post_ID, 'tracktrace_id', $track_and_trace_ID );
			$return['track_and_trace_id'] = $track_and_trace_ID;
			$return['success']            = __( 'Sucessfull updated track and trace', 'woocommerce-boostr-shipping' );
		} else {
			$return['error'] = __( 'Sorry, something went wrong - Please reload page to continue with Track and Trace', 'woocommerce-boostr-shipping' );
		}

		echo json_encode( $return );
		wp_die();
	}

	public function boostr_shipping_gls_send_tracktrace_callback( $calledFromLabelMethod = false ) {
		// Only check nonce if called with AJAX.
		if ( $calledFromLabelMethod !== true ) {
			check_ajax_referer( 'boostr-shipping-order-action' );
		}

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		if ( filter_var( $_POST['post_id'], FILTER_VALIDATE_INT ) === false ) {
			return;
		}

		$post_ID = absint( $_POST['post_id'] );

		if ( $post_ID > 0 ) {
			$order = new WC_Order( $post_ID );

			$track_and_trace_ID  = get_post_meta( $post_ID, 'tracktrace_id', true );
			$track_and_trace_URL = 'https://gls-group.eu/DK/da/find-pakke?match=' . rawurlencode( $track_and_trace_ID );
			if ( ! empty( $track_and_trace_ID ) ) {
				$Customer_email = $order->get_billing_email();
				$subject        = get_bloginfo() . ' - ' . __( 'Track and Trace', 'woocommerce-boostr-shipping' );
				$mailer         = WC()->mailer();
				ob_start();
				do_action( 'woocommerce_email_header', __( 'Track and Trace', 'woocommerce-boostr-shipping' ) . ' - #' . $order->get_id(), $mailer );
				echo esc_html( __( 'Dear, ', 'woocommerce-boostr-shipping' ) . $order->get_billing_first_name() . ' ' . $order->get_billing_last_name() );
				echo '<br/>';
				echo '<br/>';
				echo wp_kses_post( get_option( 'boostr_shipping_gls_settings' )['custom_tracktrace_email'] );
				echo '<br/>';
				echo '<br/>';
				echo esc_html( __( 'Track And Trace ID: ', 'woocommerce-boostr-shipping' ) . $track_and_trace_ID );
				echo '<br/>';
				echo '<a href="' . esc_url( $track_and_trace_URL ) . '">' . esc_html__( 'Track And Trace link', 'woocommerce-boostr-shipping' ) . '</a>';

				$aTraceTrace_instance = array(
					'date'         => date( 'd-m-Y' ),
					'time'         => date( 'H:i:s' ),
					'tracktraceID' => $track_and_trace_ID,
				);

				$tracktrace_history = array();
				array_push( $tracktrace_history, $aTraceTrace_instance );

				if ( ! add_post_meta( $post_ID, 'tracktrace_history_json', json_encode( $tracktrace_history ), true ) ) {
					$tempTrackTraceArray = json_decode( get_post_meta( $post_ID )['tracktrace_history_json'][0] );
					array_push( $tempTrackTraceArray, $aTraceTrace_instance );
					update_post_meta( $post_ID, 'tracktrace_history_json', json_encode( $tempTrackTraceArray ) );
				}
				$return['tracktrace_history_json'] = $aTraceTrace_instance;
				do_action( 'woocommerce_email_footer', $mailer );
				$message = ob_get_contents();
				ob_clean();

				if ( $mailer->send( $Customer_email, $subject, $message ) ) {
					if ( ! add_post_meta( $post_ID, 'tracktrace_mail_last_attempt', 'true', true ) ) {
						update_post_meta( $post_ID, 'tracktrace_mail_last_attempt', 'true' );
					}
					$return['success'] = __( 'Sucessfull send the mail', 'woocommerce-boostr-shipping' );
				} else {
					if ( ! add_post_meta( $post_ID, 'tracktrace_mail_last_attempt', 'false', true ) ) {
						update_post_meta( $post_ID, 'tracktrace_mail_last_attempt', 'false' );
					}
					$return['error'] = __( 'Sorry, something went wrong - wp_mail()', 'woocommerce-boostr-shipping' );
				}
			} else {
				$return['error'] = __( 'Sorry, something went wrong - Track & Trace', 'woocommerce-boostr-shipping' );
			}
		} else {
			$return['error'] = __( 'Sorry, something went wrong - post_id', 'woocommerce-boostr-shipping' );
		}
		if ( $calledFromLabelMethod != true ) {
			echo json_encode( $return );
			wp_die();
		}
	}

	public function admin_order_tracktrace_column( $columns ) {
		$columns['tracktraceStatus'] = __( 'Track & Trace', 'woocommerce-boostr-shipping' );
		return $columns;
	}

	public function admin_add_order_overview_tracktrace_column( $column ) {
		global $post, $the_order;

		if ( ! empty( $the_order ) ) {
			if ( $the_order->get_id() == $post->ID ) {
				?>
				<?php
				$the_order = new WC_Order( $post->ID );
				if ( $column == 'tracktraceStatus' ) {

					if ( ! empty( get_post_meta( $post->ID )['tracktrace_mail_last_attempt'][0] ) && get_post_meta( $post->ID )['tracktrace_mail_last_attempt'][0] ) {

						?>

						<img class="tips view"
							src="<?php echo esc_url( BOOSTR_SHIPPING_PLUGIN_URL . '/assets/images/plus-Transparent.png' ); ?>"
							alt="<?php esc_attr_e( 'Track & Trace send', 'woocommerce-boostr-shipping' ); ?>" height="14"
							width="14"></img>
						<span class="tips view"> - <?php echo esc_html( get_post_meta( $post->ID )['tracktrace_id'][0] ); ?> </span>
						<?php
					} else {

						?>
						<img class="tips view"
							src="<?php echo esc_url( BOOSTR_SHIPPING_PLUGIN_URL . '/assets/images/minus-transparent.png' ); ?>"
							alt="<?php esc_attr_e( 'Track & Trace waiting to be send', 'woocommerce-boostr-shipping' ); ?>"
							width="14"></img>
						<?php
					}
				}
			}
		}
	}

	/**
	 * Adds a metabox to the main column on the Post and Page edit screens.
	 */
	public function boostr_shipping_add_meta_box_gls_label() {
		$screens = array( 'shop_order' );
		foreach ( $screens as $screen ) {
			add_meta_box(
				'gls_get_label',
				__( 'Shipping Label', 'woocommerce-boostr-shipping' ),
				array( $this, 'boostr_shipping_gls_order_admin_html_callback' ),
				$screen,
				'side'
			);
		}
	}

	/**
	 * The method that creates the HTML for the meta box, where you generate/download the labels attached to an order
	 */
	public function boostr_shipping_gls_order_admin_html_callback( $post ) {
		$order = new WC_Order( $post->ID );
		if ( Boostr_Shipping_Admin::check_order_shipping_method( $order ) ) {
			global $the_order, $woocommerce;
			$weight = Boostr_Shipping_Admin::get_order_weight( $post->ID );

			if ( empty( $weight ) || $weight == null ) {
				$weight = 1;
			}

			$nonce = wp_create_nonce( 'boostr_shipping_dl_label' );

			$label_file_exists = false;
			$labels_upload_dir = Boostr_Shipping_Admin::get_uploads_dir();

			if ( file_exists( $labels_upload_dir . '/' . $post->ID . '.pdf' ) ) {
				$label_file_exists = true;
			}

			$query_params = array(
				'gls_label_dl' => '1',
				'_wpnonce'     => $nonce,
			);

			$filename = add_query_arg( $query_params );

			$return_label_file_exists = false;

			if ( file_exists( $labels_upload_dir . '/' . $post->ID . '_return.pdf' ) ) {
					$return_label_file_exists = true;
			}

			$query_params = array(
				'gls_label_dl' => '1',
				'return_label' => '1',
				'_wpnonce'     => $nonce,
			);

			$filename_return_label = add_query_arg( $query_params );
			?>

			<input type="hidden" name="post_id" value="<?php echo esc_attr( $post->ID ); ?>" id="gls_post_id">
			<table>
				<tr>
					<div class="wide">
						<label for="gls_weight">
							<td>
							<?php esc_html_e( 'Shipping weight', 'woocommerce-boostr-shipping' ); ?>
							</td>
							<td>
								<input class="boostr_shipping_weight_input" type="text" name="gls_weight" value="<?php echo esc_attr( $weight ); ?>" id="gls_weight" size="11">
							</td>
						</label>
					</div>
				</tr>
				<tr>
					<div class="wide">
						<label for="gls_ship_date">
							<td>
								<?php esc_html_e( 'Shipping date', 'woocommerce-boostr-shipping' ); ?>
							</td>
							<td>
							<input class="boostr_shipping_date_input" type="calendar" id="gls_ship_date" name="gls_ship_date" size="11" placeholder="dd-mm-yyyy"/>
							</td>
						</label>
					</div>
				</tr>
				<tr>
					<td>
						<input type="button" id="get_gls_label" class="button get_label button-primary" value="<?php echo esc_attr__( 'Generate label', 'woocommerce-boostr-shipping' ); ?>">
					</td>
				</tr>
				<tr class="gls-actions">
					<td>
					<a style="
					<?php
					if ( ! $label_file_exists ) {
						echo 'display:none;';}
					?>
						" href="<?php echo esc_url( $filename ); ?>" target="_blank" class="button download_label button-primary" id="download_gls_label"/><?php echo esc_html__( 'Download label', 'woocommerce-boostr-shipping' ); ?></a>
					</td>
					<td>
						<a id="download_return_label" style="
						<?php
						if ( ! $return_label_file_exists ) {
							echo 'display:none;';}
						?>
						" href="<?php echo esc_url( $filename_return_label ); ?>" target="_blank"><?php print esc_html__( 'Download return label', 'woocommerce-boostr-shipping' ); ?></a>
					</td>

				</tr>
			</table>
			<?php
		} else {
			esc_html_e( 'This order is not a order type supported by the plugin', 'woocommerce-boostr-shipping' );
		}
	}

	/**
	 * Get label from GLS
	 **/
	public function gls_label( $order_id, $date = false, $weight = 0.1, $method = 'boostr_shipping_gls_pickup', $return_label = false ) {
		if ( $weight == 0 ) {
			$weight = 0.1;
		}

		$orderdata = new WC_Order( $order_id );
		$options   = get_option( 'boostr_shipping_gls_settings' );

		$country_code = Boostr_Shipping_Admin::get_iso_3166( $orderdata->get_shipping_country() );

		if ( ! absint( $country_code ) || is_array( $country_code ) ) {
			$error = $country_code;
			return $error;
		}

		if ( ! $date ) {
			$dateTime = new DateTime();
			//$date = new DateTime('+1 day');
			//$tomorrow = $date->format('Ymd');
			//$today = date('Ymd');
		} else {
			$dateTime = new DateTime( $date );
		}

		$shipmentDate = $dateTime->format( 'Ymd' );

		$parcel_shop = get_post_meta( $orderdata->get_id(), 'shop_id' );

		$order = (object) array(
			'UserName'     => get_option( 'boostr_shipping_gls_settings' )['gls_username'],
			'Password'     => get_option( 'boostr_shipping_gls_settings' )['gls_password'],
			'Customerid'   => get_option( 'boostr_shipping_gls_settings' )['gls_customer_id'],
			'Contactid'    => get_option( 'boostr_shipping_gls_settings' )['gls_contact_id'],
			'ShipmentDate' => $shipmentDate,
			'Reference'    => '#' . $orderdata->get_order_number(),
			'Addresses'    => (object) array(
				'Delivery' => (object) array(
					'Name1'      => $orderdata->get_shipping_first_name() . ' ' . $orderdata->get_shipping_last_name(),
					'Name2'      => empty( $orderdata->get_shipping_company() ) ? $orderdata->get_billing_company() : $orderdata->get_shipping_company(),
					'Name3'      => $orderdata->get_shipping_address_2(),
					'Street1'    => $orderdata->get_shipping_address_1(),
					'CountryNum' => $country_code, //DK = 208
					'ZipCode'    => $orderdata->get_shipping_postcode(),
					'City'       => $orderdata->get_shipping_city(),
					'Contact'    => $orderdata->get_billing_first_name() . ' ' . $orderdata->get_billing_last_name(),
					'Email'      => $orderdata->get_billing_email(),
					'Phone'      => $orderdata->get_billing_phone(),
					'Mobile'     => $orderdata->get_billing_phone(),
				),
			),
			'Parcels'      => array(
				0 => (object) array(
					'Weight'    => $weight,
					'Reference' => '#' . $orderdata->get_order_number(),
					'Comment'   => $orderdata->get_customer_note(),
				),
			),
			'Services'     => (object) array(
				'Shopdelivery'      => $parcel_shop[0],
				'NotificationEmail' => $orderdata->get_billing_email(),
			),
		);

		//return label
		if ( $return_label === true ) {
			$order->Addresses->Delivery->Name1      = $options['custom_return_name'];
			$order->Addresses->Delivery->Street1    = $options['custom_return_street'];
			$order->Addresses->Delivery->CountryNum = $options['custom_return_countrynum'];
			$order->Addresses->Delivery->ZipCode    = $options['custom_return_zipcode'];
			$order->Addresses->Delivery->City       = $options['custom_return_city'];

			$order->Addresses->Delivery->Contact = '';
			$order->Addresses->Delivery->Email   = '';
			$order->Addresses->Delivery->Phone   = '';
			$order->Addresses->Delivery->Mobile  = '';

			$order->Addresses->Pickup->Name1      = $orderdata->get_shipping_first_name() . ' ' . $orderdata->get_shipping_last_name();
			$order->Addresses->Pickup->Name2      = empty( $orderdata->get_shipping_company() ) ? $orderdata->get_billing_company() : $orderdata->get_shipping_company();
			$order->Addresses->Pickup->Street1    = $orderdata->get_shipping_address_1();
			$order->Addresses->Pickup->CountryNum = $country_code;
			$order->Addresses->Pickup->ZipCode    = $orderdata->get_shipping_postcode();
			$order->Addresses->Pickup->City       = $orderdata->get_shipping_city();

			if ( $method = 'boostr_shipping_gls_pickup' ) {
				$order->Addresses->Pickup->Street1    = $orderdata->get_billing_address_1();
				$order->Addresses->Pickup->CountryNum = $country_code;
				$order->Addresses->Pickup->ZipCode    = $orderdata->get_billing_postcode();
				$order->Addresses->Pickup->City       = $orderdata->get_billing_city();
				$order->Addresses->Delivery->Name2    = '';
				$order->Addresses->Delivery->Name3    = '';
			}

			$order->Services->ShopReturn = 'Y';

		}

		if ( $method === 'boostr_shipping_gls_private' ) {

			$country_exceptions = array( 'SE', 'GL', 'FO' );
			if ( ! in_array( $orderdata->get_shipping_country(), $country_exceptions ) ) {
				$order->Services->PrivateDelivery = 'Y';
			}
		}

		//insert other shipping methods
		if ( $method == 'boostr_shipping_gls_private' || $method == 'boostr_shipping_gls_business' || $return_label ) {
			unset( $order->Services->Shopdelivery );
			$order->Services->AddresseeOnly = true;
		}

		//gets the label from GLS
		try {
			$json_order = wp_json_encode( $order );

			$response = wp_safe_remote_post(
				'https://api.gls.dk/ws/DK/V1/CreateShipment',
				array(
					'body'    => $json_order,
					'headers' => array(
						'Content-Type' => 'application/json',
					),
				)
			);

			if ( ! is_wp_error( $response ) ) {
				$response = wp_remote_retrieve_body( $response );
				$response = json_decode( $response );
			}

			if ( ! is_wp_error( $response ) && isset( $response->PDF ) ) {
				$order_note_message = __( 'Order Label exported', 'woocommerce-boostr-shipping' );
				$orderdata->add_order_note( $order_note_message );

				return $response;
			} else {
				return $response->Message;
			}
			exit;
		} catch ( Exception $e ) {
			return 'Caught exception: ' . $e->getMessage() . "\n";
		}
	}

	public function gls_get_label_callback() {
		check_ajax_referer( 'boostr-shipping-order-action' );

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		if ( filter_var( $_POST['post_id'], FILTER_VALIDATE_INT ) === false ) {
			wp_die();
		}

		if ( filter_var( str_replace( ',', '.', $_POST['gls_ship_weight'] ), FILTER_VALIDATE_FLOAT ) === false ) {
			wp_die();
		}

		$post_id = absint( $_POST['post_id'] );

		if ( $post_id > 0 ) {
			$date   = $_POST['gls_ship_date'];
			$weight = (float) str_replace( ',', '.', $_POST['gls_ship_weight'] );
			$order  = new WC_Order( $post_id );
			$method = Boostr_Shipping_Admin::check_order_shipping_method( $order );

			if ( $method === 'boostr_shipping_gls_pickup' && $weight > 20 ) {
				wp_send_json( array( 'error' => __( 'The order weight is higher than the allowed 20 kg for ParcelShop deliveries', 'woocommerce-boostr-shipping' ) ) );
			} elseif ( $weight > 30 ) {
				wp_send_json( array( 'error' => __( 'The order weight is higher than the allowed 30 kg for private and business deliveries', 'woocommerce-boostr-shipping' ) ) );
			}

			$options = get_option( 'boostr_shipping_gls_settings' );

			$returned = $this->gls_label( $post_id, $date, $weight, $method );

			if ( $returned->PDF ) {
				if ( Boostr_Shipping_Admin::create_uploads_files() ) {
					$labels_upload_dir = Boostr_Shipping_Admin::get_uploads_dir();
					$file              = $labels_upload_dir . '/' . $post_id . '.pdf';

					$query_params  = array( 'gls_label_dl' => '1' );
					$file_location = add_query_arg( $query_params, get_edit_post_link( $post_id ) );

					$content = base64_decode( $returned->PDF );
					file_put_contents( $file, $content );
					$return['file'] = $file_location;
					$tracktrace_id  = $returned->ConsignmentId;

					if ( ! add_post_meta( $post_id, 'tracktrace_id', $tracktrace_id, true ) ) {
						update_post_meta( $post_id, 'tracktrace_id', $tracktrace_id );
					}

					if ( get_option( 'boostr_shipping_gls_settings' )['auto_tracktrace'] == 'Yes' ) {
						$this->boostr_shipping_gls_send_tracktrace_callback( true );
					}
				} else {
					$return['error'] = __( "Couldn't save label to file", 'woocommerce-boostr-shipping' );
				}
			} else {
				$return['error'] = $returned;
			}

			//return label
			if ( ! empty( $options['enable_return_label'] ) ) {

				if ( $options['enable_return_label'] == 'Yes' ) {
					$returned_return_label = $this->gls_label( $post_id, $date, $weight, $method, true );

					if ( isset( $returned_return_label->PDF ) ) {
						if ( Boostr_Shipping_Admin::create_uploads_files() ) {
							$labels_upload_dir = Boostr_Shipping_Admin::get_uploads_dir();
							$file              = $labels_upload_dir . '/' . $post_id . '_return.pdf';

							$query_params  = array(
								'gls_label_dl' => '1',
								'return_label' => '1',
							);
							$file_location = add_query_arg( $query_params, get_edit_post_link( $post_id ) );

							$content = base64_decode( $returned_return_label->PDF );
							file_put_contents( $file, $content );
							$return['file_return_label'] = $file_location;
						} else {
							$return['error'] = __( "Couldn't save label to file", 'woocommerce-boostr-shipping' );
						}
					} else {
						$return['retun_label_error'] = $returned_return_label;
					}
				}
			}
		} else {
			$return['error'] = __( 'Sorry, something went wrong with the Order ID', 'woocommerce-boostr-shipping' );
		}
		wp_send_json( $return );
	}

	public function add_order_actions( $actions, $order ) {
		$order_id        = $order->get_id();
		$shipping_method = Boostr_Shipping_Admin::check_order_shipping_method( $order );

		if ( $shipping_method === false ) {
			return $actions;
		}

		$paged = ( isset( $_GET['paged'] ) ) ? '&paged=' . absint( $_GET['paged'] ) : '';

		$actions['export_gls_ftp'] = array(
			'url'    => wp_nonce_url( admin_url( '?order_id=' . $order_id . '&action=export_gls_ftp' . $paged ), 'export-gls-ftp' ),
			'name'   => __( 'Export Label to FTP', 'woocommerce-boostr-shipping' ),
			'action' => 'export-gls-ftp',
		);

		$actions['export_gls_file'] = array(
			'url'    => wp_nonce_url( admin_url( '?order_id=' . $order_id . '&action=export_gls_file' ), 'export-gls-file' ),
			'name'   => __( 'Export Label to File', 'woocommerce-boostr-shipping' ),
			'action' => 'export-gls-file',
		);
		return $actions;
	}

	/**
	 * Make the file ready with all the data and then pass it on to ftp or send to user
	 **/
	public function make_export_file_ready( $post_ids ) {
		$exported            = 0;
		$fileContent         = '';
		$seperator           = ',';
		$exported_error      = array();
		$options             = get_option( 'boostr_shipping_gls_settings' );
		$enable_return_label = ( ! empty( $options['enable_return_label'] ) ) ? $options['enable_return_label'] : 'No';

		if ( $post_ids && is_array( $post_ids ) && count( $post_ids ) > 0 ) {
			foreach ( $post_ids as $post_id ) {
				$orderdata = new WC_Order( $post_id );

				$shipping_method = Boostr_Shipping_Admin::check_order_shipping_method( $orderdata );

				if ( $shipping_method === false ) {
					$exported_error[] = $post_id;
					continue;
				}

				$weight        = ( Boostr_Shipping_Admin::get_order_weight( $post_id ) ? Boostr_Shipping_Admin::get_order_weight( $post_id ) : 0.1 );
				$today         = new DateTime();
				$services      = '';
				$parcelshop_id = '';
				$parcel_type   = 'A';

				if ( $shipping_method === 'boostr_shipping_gls_pickup' ) {
					$shipment_type = 'Z'; // Parcelshop delivery.

					if ( $enable_return_label === 'Yes' ) {
						$services = 'B';
					}

					$parcelshop_id = get_post_meta( $orderdata->get_id(), 'shop_id', true );
				} elseif ( $orderdata->get_shipping_country() !== 'DK' ) {
					$shipment_type = 'U'; // Other country than DK
				} else {
					$shipment_type = 'A'; // Private/Business delivery

					if ( $enable_return_label === 'Yes' ) {
						$services = 'B';
					}
				}

				if ( $shipping_method === 'boostr_shipping_gls_private' ) {
					$parcel_type = 'M';
				}

				$fileContent .= '"' . $orderdata->get_id() . '"' . $seperator; // (Order#) //$orderdata->get_order_number()
				$fileContent .= ! empty( trim( $orderdata->get_shipping_company() ) ) ? '"' . $orderdata->get_shipping_company() . '"' . $seperator : '"' . substr( $orderdata->get_shipping_first_name() . ' ' . $orderdata->get_shipping_last_name(), 0, 28 ) . '"' . $seperator; // (Consignee Name)
				$fileContent .= '"' . $orderdata->get_shipping_address_1() . '"' . $seperator; // (Consignee Address 1)
				$fileContent .= '""' . $seperator; // (Consignee Address 2)
				$fileContent .= '"' . $orderdata->get_shipping_postcode() . '"' . $seperator; // (Consignee Postal Code)
				$fileContent .= '"' . $orderdata->get_shipping_city() . '"' . $seperator; // (Consignee City)
				$fileContent .= '"' . Boostr_Shipping_Admin::get_interline_country_code( $orderdata->get_shipping_country() ) . '"' . $seperator; // (Consignee Country)
				$fileContent .= '"' . $today->format( 'd-m-y' ) . '"' . $seperator; // (Date (DD-MM-YY) )
				$fileContent .= '"' . $weight . '"' . $seperator; // (Parcel weight)
				$fileContent .= '"1"' . $seperator; // (Number of parcels)
				$fileContent .= '""' . $seperator; // Not in use
				$fileContent .= '""' . $seperator; // (Parcel value amount)
				$fileContent .= '"' . $parcel_type . '"' . $seperator; // (Parcel type A = Normal parcel, S = Small parcel, M = PrivateDelivery)
				$fileContent .= '"' . $shipment_type . '"' . $seperator; // (Shipment type A = Normal distribution U = Other country than DK Z = ShopDeliveryService 2 = ShopReturnService mail B = ShopReturnService label 4 = Express10Service 5 = Express12Service)
				$fileContent .= ! empty( trim( $orderdata->get_shipping_company() ) ) ? '"' . substr( $orderdata->get_shipping_first_name() . ' ' . $orderdata->get_shipping_last_name(), 0, 28 ) . '"' . $seperator : '""' . $seperator; //(Att)
				$fileContent .= '"' . substr( $orderdata->get_customer_note(), 0, 28 ) . '"' . $seperator; // (Comment)
				$fileContent .= '""' . $seperator; // Forsendelse
				$fileContent .= '"' . $orderdata->get_billing_email() . '"' . $seperator; // (Consignee mail address)
				$fileContent .= '"' . $orderdata->get_billing_phone() . '"' . $seperator; // (Consignee mobile phone no.)
				$fileContent .= '"' . $services . '"' . $seperator; // (Services: E = Mail notification, B = ShopReturnService label)
				$fileContent .= '""' . $seperator; // pxx = printer no. (21)
				$fileContent .= '"' . $parcelshop_id . '"'; // ParcelShop ID (22)

				$fileContent .= "\r\n";

				$exported++;

			}
		}

		return array(
			'csv_content'    => $fileContent,
			'exported'       => $exported,
			'exported_error' => $exported_error,
		);
	}

	public function serve_csv_file( $csv_content ) {
		ob_start();
		header( 'Content-Encoding: UTF-8' );
		header( 'Content-type: text/csv;charset=UTF-8' );
		header( 'Content-Disposition: attachment; filename=gls-' . uniqid() . '.csv' );
		header( 'Pragma: no-cache' );
		header( 'Expires: 0' );
		ob_clean();
		ob_end_flush();
		echo $csv_content;
		exit;
	}

	/**
	 * Export and save CSV file to ftp
	 **/
	public function export_to_csv_ftp( $file_content ) {
		$settings = get_option( 'boostr_shipping_ftp_settings' );

		if ( empty( $settings ) ) {
			return false;
		}

		if ( isset( $settings['export_ftp_folder'] ) ) {
			$remote_file = trim( $settings['export_ftp_folder'] ) . '/gls-labels.csv';
		} else {
			$remote_file = '/gls-labels.csv';
		}

		if ( ! isset( $settings['export_ftp_address'], $settings['export_ftp_port'] ) ) {
			return false;
		}

		$ftp_username = $settings['export_ftp_username'];
		$ftp_password = $settings['export_ftp_password'];
		$use_ssl      = true;

		if ( isset( $settings['export_ftp_use_ssl'] ) && $settings['export_ftp_use_ssl'] === 'No' ) {
			$use_ssl = false;
		}

		$login_result = false;

		if ( $use_ssl === true ) {
			if ( ! function_exists( 'ftp_ssl_connect' ) ) {
				return false;
			}

			$conn_id = ftp_ssl_connect( $settings['export_ftp_address'], absint( $settings['export_ftp_port'] ) );
		} else {
			$conn_id = ftp_connect( $settings['export_ftp_address'], absint( $settings['export_ftp_port'] ) );
		}

		if ( $conn_id !== false || $conn_id !== null ) {
			$login_result = ftp_login( $conn_id, $ftp_username, $ftp_password );
		} else {
			return false;
		}

		if ( $login_result === false ) {
			ftp_close( $conn_id );
			return false;
		}

		ftp_pasv( $conn_id, true );

		// Create temporary csv file for upload.
		$csv_file = fopen( 'php://temp', 'r+' );
		fputs( $csv_file, $file_content );
		rewind( $csv_file );

		// Upload the file
		if ( ftp_fput( $conn_id, $remote_file, $csv_file, FTP_BINARY ) ) {
			$res = true;
		} else {
			$res = false;
		}

		// Close the connection
		ftp_close( $conn_id );

		return $res;
	}

	public function add_bulk_actions( $bulk_actions ) {
		$bulk_actions['exporttoftp']  = __( 'Export to ftp', 'woocommerce-boostr-shipping' );
		$bulk_actions['exporttofile'] = __( 'Export to file', 'woocommerce-boostr-shipping' );

		return $bulk_actions;
	}

	public function export_bulk_admin_notices() {
		global $post_type, $pagenow;

		if ( $pagenow == 'edit.php' && $post_type == 'shop_order' && isset( $_REQUEST['exported'] ) && (int) $_REQUEST['exported'] && ( ! isset( $_REQUEST['ftperror'] ) || $_REQUEST['ftperror'] === '0' ) ) {
			$message = sprintf( _n( 'Order exported to ftp.', '%s orders exported to ftp.', $_REQUEST['exported'], 'woocommerce-boostr-shipping' ), number_format_i18n( $_REQUEST['exported'] ) );
			echo "<div class='updated'><p>" . esc_html( $message ) . '</p></div>';
		}
		if ( $pagenow == 'edit.php' && $post_type == 'shop_order' && isset( $_REQUEST['ftperror'] ) && $_REQUEST['ftperror'] === '1' ) {
			$message = __( 'There was a problem connecting to the ftp.', 'woocommerce-boostr-shipping' );
			echo "<div class='error'><p>" . esc_html( $message ) . '</p></div>';
		}
		if ( $pagenow == 'edit.php' && $post_type == 'shop_order' && isset( $_REQUEST['export_error'] ) && $_REQUEST['export_error'] === '1' ) {
			$message = _n( 'There was an error exporting the order', 'There was a problem exporting the orders.', $_REQUEST['exported'], 'woocommerce-boostr-shipping' );
			echo "<div class='error'><p>" . esc_html( $message ) . '</p></div>';
		}
		if ( $pagenow == 'edit.php' && $post_type == 'shop_order' && isset( $_REQUEST['exported_error'] ) && absint( $_REQUEST['exported_error'] ) ) {
			$message      = __( 'There was an error exporting: ', 'woocommerce-boostr-shipping' );
			$orders_error = '#' . implode( ', #', $_REQUEST['exported_error'] );
			$message_2    = _n( 'This order is not a GLS Packetshop order!', 'These orders are not GLS Packetshop orders!', count( $_REQUEST['exported_error'] ), 'woocommerce-boostr-shipping' );
			echo "<div class='error'><p>" . esc_html( $message ) . '<b>' . esc_html( $orders_error ) . '</b><br />' . esc_html( $message_2 ) . '</p></div>';
		}
	}

	public function export_bulk_actions( $redirect_url, $action, $post_ids ) {
		$allowed_actions = array( 'exporttoftp', 'exporttofile' );

		if ( ! in_array( $action, $allowed_actions ) ) {
			return $redirect_url;
		}

		// security check
		check_admin_referer( 'bulk-posts' );

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		switch ( $action ) {
			case 'exporttoftp':
				$export_error = 0;
				$returned     = $this->make_export_file_ready( $post_ids );

				if ( $returned['exported'] == 0 ) {
					$export_error = 1;
				}

				$ftp_res      = $this->export_to_csv_ftp( $returned['csv_content'] );
				$redirect_url = add_query_arg(
					array(
						'exported'       => $returned['exported'],
						'export_error'   => $export_error,
						'ftperror'       => $ftp_res === true ? '0' : '1',
						'exported_error' => $returned['exported_error'],
						'ids'            => join(
							',',
							$post_ids
						),
					),
					$redirect_url
				);
				break;
			case 'exporttofile':
				$returned = $this->make_export_file_ready( $post_ids );
				$this->serve_csv_file( $returned['csv_content'] );
				break;
			default:
				return $redirect_url;
		}

		return $redirect_url;
	}

	public function export_csv_file_action() {
		if ( ! isset( $_GET['action'], $_GET['_wpnonce'], $_GET['order_id'] ) || $_GET['action'] !== 'export_gls_file' ) {
			return;
		}

		if ( ! wp_verify_nonce( $_GET['_wpnonce'], 'export-gls-file' ) || ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		if ( filter_var( $_GET['order_id'], FILTER_VALIDATE_INT ) === false ) {
			return;
		}

		$order_id = (int) $_GET['order_id'];

		$res = $this->make_export_file_ready( (array) $order_id );
		$this->serve_csv_file( $res['csv_content'] );
	}

	public function export_csv_ftp_action() {
		if ( ! isset( $_GET['action'], $_GET['_wpnonce'], $_GET['order_id'] ) || $_GET['action'] !== 'export_gls_ftp' ) {
			return;
		}

		if ( ! wp_verify_nonce( $_GET['_wpnonce'], 'export-gls-ftp' ) || ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die();
		}

		if ( filter_var( $_GET['order_id'], FILTER_VALIDATE_INT ) === false ) {
			return;
		}

		$order_id = (int) $_GET['order_id'];

		$returned     = $this->make_export_file_ready( (array) $order_id );
		$ftp_res      = $this->export_to_csv_ftp( $returned['csv_content'] );
		$export_error = 0;

		if ( $returned['exported'] == 0 ) {
			$export_error = 1;
		}

		$paged = ( isset( $_GET['paged'] ) ) ? '&paged=' . absint( $_GET['paged'] ) : '';

		$sendback = add_query_arg(
			array(
				'exported'       => $returned['exported'],
				'export_error'   => $export_error,
				'ftperror'       => $ftp_res === true ? '0' : '1',
				'exported_error' => $returned['exported_error'],
			),
			admin_url( 'edit.php?post_type=shop_order' . $paged )
		);

		wp_safe_redirect( $sendback );
		exit;
	}

	public function download_label() {
		if ( ! isset( $_GET['gls_label_dl'], $_GET['post'], $_GET['_wpnonce'] ) ) {
			return;
		}

		if ( ! wp_verify_nonce( $_GET['_wpnonce'], 'boostr_shipping_dl_label' ) ) {
			wp_die();
		}

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			return;
		}

		if ( false === filter_var( $_GET['post'], FILTER_VALIDATE_INT ) ) {
			return;
		}

		$post_id = $_GET['post'];

		global $pagenow;
		if ( ( $pagenow !== 'post.php' ) ) {
			return;
		}

		if ( ! current_user_can( 'edit_shop_orders' ) ) {
			exit;
		}

		$labels_upload_dir = Boostr_Shipping_Admin::get_uploads_dir();
		// Get the file path for the label / return label.
		if ( isset( $_GET['return_label'] ) ) {
			$file = $labels_upload_dir . '/' . $post_id . '_return.pdf';
		} else {
			$file = $labels_upload_dir . '/' . $post_id . '.pdf';
		}

		$content_disposition = 'inline';

		if ( file_exists( $file ) ) {
			ob_start();
			header( 'Content-Type: application/pdf' );
			header( 'Content-Disposition: ' . $content_disposition . '; filename="' . basename( $file ) . '"' );
			header( 'Content-Length: ' . filesize( $file ) );
			ob_clean();
			ob_end_flush();
			readfile( $file );
			exit;
		}
	}
}
