<?php

defined( 'ABSPATH' ) || exit;

abstract class Boostr_Shipping_Main extends WC_Shipping_Method {
	abstract protected function register_shipping_method( $methods );
	abstract protected function addActions();
	abstract protected function addFilters();

	public function init() {
		$this->instance_form_fields      = $this->default_settings();
		$this->title                     = $this->get_option( 'title' );
		$this->tax_status                = $this->get_option( 'tax_status' );
		$this->shipping_price            = $this->get_option( 'shipping_price' );
		$this->enable_free_shipping      = $this->get_option( 'enable_free_shipping' );
		$this->free_shipping_total       = $this->get_option( 'free_shipping_total' );
		$this->differentiated_price_type = $this->get_option( 'differentiated_price_type' );

		$this->init_settings();

		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
		add_filter( 'body_class', array( $this, 'boostr_body_class' ) );
	}

	public function mainAddAction() {
	}

	public function boostr_body_class( $classes ) {
		$theme_data = wp_get_theme();
		if ( ! empty( $theme_data->get( 'Template' ) ) ) {
			$classes[] = 'boostr-active-theme-is-' . $theme_data->get( 'Template' );
		} else {
			$classes[] = 'boostr-active-theme-is-' . $theme_data->get( 'Name' );
		}
		return $classes;
	}

	public function default_settings() {
		$settings_default = array(
			'hidden_post_field'         => array(
				'type'  => 'hidden',
				'class' => 'hidden_post_field',
			),
			'title'                     => array(
				'title'       => __( 'Method Name', 'woocommerce-boostr-shipping' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the costumer sees during checkout.', 'woocommerce-boostr-shipping' ),
				'default'     => $this->method_title,
				'desc_tip'    => true,
			),
			'tax_status'                => array(
				'title'   => __( 'Tax Status', 'woocommerce-boostr-shipping' ),
				'type'    => 'select',
				'class'   => 'wc-enhanced-select',
				'default' => 'taxable',
				'options' => array(
					'taxable' => __( 'Taxable', 'woocommerce-boostr-shipping' ),
					'none'    => _x( 'None', 'Tax status', 'woocommerce-boostr-shipping' ),
				),
			),
			'enable_free_shipping'      => array(
				'title'   => __( 'Enable free shipping', 'woocommerce-boostr-shipping' ),
				'type'    => 'select',
				'class'   => 'boostr-Shipping-select-enable',
				'default' => 'taxable',
				'options' => array(
					'No'  => __( 'No', 'woocommerce-boostr-shipping' ),
					'Yes' => __( 'Yes', 'woocommerce-boostr-shipping' ),
				),
			),
			'free_shipping_total'       => array(
				'title'       => __( 'Minimum price for free shipping', 'woocommerce-boostr-shipping' ),
				'type'        => 'text',
				'class'       => 'boostr-Shipping-minimum-price',
				'description' => __( 'If free shipping have been enabled this will, control the minimum amount that the customer will have to spend for free shipping', 'woocommerce-boostr-shipping' ),
				'default'     => 0,
				'desc_tip'    => true,
			),
			'differentiated_price_type' => array(
				'title'       => __( 'Differentiated Price Type', 'woocommerce-boostr-shipping' ),
				'type'        => 'select',
				'description' => __( 'Choose what the shipping price is based on', 'woocommerce-boostr-shipping' ),
				'default'     => 'Normal',
				'class'       => 'differentiated_price_type',
				'options'     => array(
					'Quantity' => __( 'Normal', 'woocommerce-boostr-shipping' ),
					'Weight'   => __( 'Weight', 'woocommerce-boostr-shipping' ),
					'Price'    => __( 'Price', 'woocommerce-boostr-shipping' ),
				),
			),
			'shipping_price'            => array(
				'title'       => __( 'Shipping Price', 'woocommerce-boostr-shipping' ),
				'type'        => 'text',
				'description' => __( 'This controls what the costumer will have to pay for this shipping method.<br/><br/> Simple math functions can also be inserted instead, and will be calculated.<br/><br/>Using [qty] is a placeholder for the quantity of items in the cart', 'woocommerce-boostr-shipping' ),
				'class'       => 'shipping_price',
				'default'     => 0,
				'desc_tip'    => true,
			),
		);

		return $settings_default;
	}

	/* Returns/Echo's the HTML that is used to make a zipcode textarea & the find shop button */
	public function HTML_zipAndFind( $sShipping_type ) {
		if ( is_checkout() ) {
			?>
				<div class="boostr_shipping_field-wrap">
					<div class="boostr_shipping_shipping_field">
						<div class="boostr-clearfix" id="boostr_shipping_button">
							<div class="boostr-shipping-stores ">
								<input type="text"
									id="boostr_shipping_shop_zip"
									<?php
									if ( isset( $_SESSION['zipcode'] ) && $_SESSION['zipcode'] ) {
										echo ' value="' . esc_attr( $_SESSION['zipcode'] ) . '"';
									}
									?>
									name="boostr_shipping_shop_zip" class="input"
									placeholder="<?php echo esc_attr__( 'Zipcode', 'woocommerce-boostr-shipping' ); ?>">
								<input type="button" id="boostr_shipping_shop_button" name="boostr_shipping_shop"
									class="button"
									onClick="getShopList('<?php echo esc_js( $sShipping_type ); ?>', jQuery('#boostr_shipping_shop_zip').val());"
									value="<?php echo esc_attr__( 'Find nearest pickup point', 'woocommerce-boostr-shipping' ); ?>">
							</div>
						</div>
						<div id="hidden_choosen_shop" type="hidden"></div>
						<div id="selected_shop_wrapper">
							<div class="boostr-clearfix" id="selected_shop_header">
							<?php echo esc_html__( ' No pickup point have been choosen!', 'woocommerce-boostr-shipping' ); ?>
							</div>
							<div class="boostr-clearfix" id="selected_shop_context">

							</div>
							<div id="boostr_shipping_hidden_radio">

							</div>
						</div>
					</div>
				</div>
				<div id="shops-list"></div>
				<script>
					jQuery(document).ready(function () {

						jQuery(document).on('keypress', function (event) {
							if (jQuery('.shop-list').hasClass('selected_single_shop') && event.keyCode == 13 && jQuery('#boostr-shipping-modal:visible').length != 0) {
								jQuery('#boostr-choose-shop-btn').trigger("click");
								jQuery('#boostr-choose-shop-btn').blur();
							}
						});

						jQuery('#boostr_shipping_shop_button').on('click', function () {
							jQuery(' div#order_review > table > tfoot > .shipping > td').css('position', 'relative').append('<div class="blockUI blockOverlay boostr-shipping-loader"></div>');
						})


						jQuery('#boostr_shipping_shop_zip').on('keypress', function (event) {
							if (event.keyCode == 13) {
								event.preventDefault();
								jQuery('#boostr_shipping_shop_button').click();
								jQuery('#boostr_shipping_shop_zip').blur();
							}
						})

						jQuery(document).on("click input change", "#shipping_method > li > .shipping_method", function () {
							if (jQuery(this).attr('id').includes("boostr_shipping_gls_pickup")) {
								jQuery('#shipping_address_1,  #shipping_address_2, #shipping_postcode,  #shipping_city').prop('disabled', true);
							}
							else {
								jQuery('#shipping_address_1,  #shipping_address_2, #shipping_postcode,  #shipping_city').prop('disabled', false);
							}
						});
					});
				</script>
				<?php
		} else {
			echo '<br/><div class="shipping_pickup_cart">' . esc_html__( 'Pickup point is selected during checkout', 'woocommerce-boostr-shipping' ) . '</div>';
		}
	}

	protected function evaluate_cost( $sum, $args = array() ) {
		include_once WC()->plugin_path() . '/includes/libraries/class-wc-eval-math.php';

		// Allow 3rd parties to process shipping cost arguments
		$args     = apply_filters( 'woocommerce_evaluate_shipping_cost_args', $args, $sum, $this ); // phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedHooknameFound
		$locale   = localeconv();
		$decimals = array( wc_get_price_decimal_separator(), $locale['decimal_point'], $locale['mon_decimal_point'], ',' );

		// Remove whitespace from string
		$sum = preg_replace( '/\s+/', '', $sum );
		// Remove locale from string
		$sum = str_replace( $decimals, '.', $sum );
		// Trim invalid start/end characters
		$sum = rtrim( ltrim( $sum, "\t\n\r\0\x0B+*/" ), "\t\n\r\0\x0B+-*/" );
		// Do the math
		return $sum ? WC_Eval_Math::evaluate( $sum ) : 0;
	}

	public function calculate_shipping( $package = array() ) {
		require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/lib/field_calculator.php';
		global $woocommerce;
		if ( wc_tax_enabled() ) {
			$iCart_total = $woocommerce->cart->cart_contents_total + array_sum( $woocommerce->cart->get_cart_contents_taxes() );
		} else {
			$iCart_total = $woocommerce->cart->cart_contents_total;
		}

		//Enable "free shipping" will overrule any other
		if ( $this->enable_free_shipping == 'Yes' && $iCart_total >= $this->free_shipping_total ) {
			$return_price = 0;
		} else {
			if ( $this->get_option( 'differentiated_price_type' ) == 'Price' ) {
				$oRows = get_option( 'Price_' . $this->instance_id );
				foreach ( $oRows as $oRow ) {
					if ( $oRow->minimum < $iCart_total && $iCart_total <= $oRow->maximum ) {
						$return_price = (float) $oRow->shipping_price;
					}
				}
				if ( ! isset( $return_price ) ) {
					$return_price = 0;
					foreach ( $oRows as $oRow ) {
						if ( (float) $oRow->shipping_price > $return_price ) {
							$return_price = (float) $oRow->shipping_price;
						}
					}
				}
			} elseif ( $this->get_option( 'differentiated_price_type' ) == 'Weight' ) {
				$iCartWeight = $woocommerce->cart->cart_contents_weight;
				$oRows       = get_option( 'Weight_' . $this->instance_id );
				foreach ( $oRows as $oRow ) {
					if ( $oRow->minimum < $iCartWeight && $iCartWeight <= $oRow->maximum ) {
						$return_price = (float) $oRow->shipping_price;
					}
				}

				if ( ! isset( $return_price ) ) {
					$return_price = 0;
					foreach ( $oRows as $oRow ) {
						if ( (float) $oRow->shipping_price > $return_price ) {
							$return_price = (float) $oRow->shipping_price;
						}
					}
				}
			} else {
				$Cal          = new Field_calculate();
				$return_price = (float) $Cal->calculate( str_replace( '[qty]', (string) WC()->cart->get_cart_contents_count(), $this->shipping_price ) );
			}
		}

		$coupons = $woocommerce->cart->applied_coupons;

		if ( ! empty( $coupons ) ) {
			foreach ( $coupons as $coupon ) {
				$applied_coupon = new WC_Coupon( $coupon );

				if ( $applied_coupon->get_free_shipping() ) {
					$return_price = 0;
					break;
				}
			}
		}

		$rateLabel    = $this->title;
		$return_price = $this->evaluate_cost( $return_price, array() );

		$rate = array(
			'label' => $rateLabel,
			'id'    => $this->id . $this->instance_id,
			'cost'  => $return_price,
		);

		$this->add_rate( $rate );
	}
}
