<?php

defined( 'ABSPATH' ) || exit;

class Boostr_Shipping_Settings {
	public function register_actions() {
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_settings_enqueue' ) );
		add_action( 'admin_menu', array( $this, 'add_admin_menu' ) );
		add_action( 'admin_init', array( $this, 'settings_init' ) );
		add_filter( 'plugin_action_links_' . BOOSTR_SHIPPING_PLUGIN_BASENAME, array( $this, 'plugin_action_links' ) );
	}

	public function admin_settings_enqueue() {
		wp_register_style( 'admin_post_style', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/css/admin/boostr-shipping-admin.css', array(), BOOSTR_SHIPPING_VERSION );
		wp_enqueue_style( 'admin_post_style' );
	}

	public function add_admin_menu() {
		add_menu_page( __( 'Plugins.dk GLS', 'woocommerce-boostr-shipping' ), __( 'Plugins.dk GLS', 'woocommerce-boostr-shipping' ), 'manage_options', 'boostr-shipping', array( $this, 'options_page' ), BOOSTR_SHIPPING_PLUGIN_URL . '/assets/images/pluginsdk-wpicon.png' );
	}

	public function settings_init() {
		register_setting( 'boostr_shipping_general_group', 'boostr_shipping_general_settings' );

		add_settings_section(
			'boostr_shipping_general',
			__( 'General Settings', 'woocommerce-boostr-shipping' ),
			array( $this, 'general_settings_section_callback' ),
			'boostr_shipping_general_group'
		);

		register_setting( 'boostr_shipping_gls_group', 'boostr_shipping_gls_settings' );

		add_settings_section(
			'boostr_shipping_gls',
			__( 'GLS settings', 'woocommerce-boostr-shipping' ),
			array( $this, 'gls_settings_section_callback' ),
			'boostr_shipping_gls_group'
		);

		add_settings_field(
			'gls_customer_id',
			__( 'Your GLS Customer ID', 'woocommerce-boostr-shipping' ),
			array( $this, 'gls_customer_id_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'gls_username',
			__( 'Your GLS Username', 'woocommerce-boostr-shipping' ),
			array( $this, 'gls_username_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'gls_password',
			__( 'Your GLS Password', 'woocommerce-boostr-shipping' ),
			array( $this, 'gls_password_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'gls_contact_id',
			__( 'Your GLS Contact ID', 'woocommerce-boostr-shipping' ),
			array( $this, 'gls_contact_id_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'auto_tracktrace',
			__( 'Automaticly email Track & Trace, when generating a label', 'woocommerce-boostr-shipping' ),
			array( $this, 'auto_tracktrace_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_tracktrace_email',
			__( 'Edit the inner part of the Track and trace email send to your customer.', 'woocommerce-boostr-shipping' ),
			array( $this, 'custom_tracktrace_email_render' ),
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_tracktrace_enable_own_tt',
			__( 'Enabel own track and trace number. This setting can only be used if the track & trace number is generated elsewhere. for example in the GLS program Interline', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				if ( empty( $options['custom_tracktrace_enable_own_tt'] ) ) {
					$options['custom_tracktrace_enable_own_tt'] = 'No';
				}
				?>
			<select name='boostr_shipping_gls_settings[custom_tracktrace_enable_own_tt]'>
					<option value="No" <?php selected( $options['custom_tracktrace_enable_own_tt'], 'No' ); ?>><?php echo esc_html__( 'No', 'woocommerce-boostr-shipping' ); ?></option>
					<option value="Yes" <?php selected( $options['custom_tracktrace_enable_own_tt'], 'Yes' ); ?>><?php echo esc_html__( 'Yes', 'woocommerce-boostr-shipping' ); ?></option>
			</select>
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'enable_return_label',
			__( 'Enabel return label', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				if ( empty( $options['enable_return_label'] ) ) {
					$options['enable_return_label'] = 'No';
				}
				?>
				<select name='boostr_shipping_gls_settings[enable_return_label]'>
					<option value="No" <?php selected( $options['enable_return_label'], 'No' ); ?>><?php echo esc_html__( 'No', 'woocommerce-boostr-shipping' ); ?></option>
					<option value="Yes" <?php selected( $options['enable_return_label'], 'Yes' ); ?>><?php echo esc_html__( 'Yes', 'woocommerce-boostr-shipping' ); ?></option>
				</select>
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_return_name',
			__( 'Returners name', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				?>
				<input id="gls_custom_return_name" type='text' name='boostr_shipping_gls_settings[custom_return_name]' value="<?php echo esc_attr( ( ! empty( $options['custom_return_name'] ) ? $options['custom_return_name'] : '' ) ); ?>">
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_return_countrynum',
			__( 'Country number', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				?>
				<input id="gls_custom_return_countrynum" type='text' name='boostr_shipping_gls_settings[custom_return_countrynum]' value="<?php echo esc_attr( ( ! empty( $options['custom_return_countrynum'] ) ? $options['custom_return_countrynum'] : '' ) ); ?>">
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_return_street',
			__( 'Address', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				?>
				<input id="gls_custom_return_street" type='text' name='boostr_shipping_gls_settings[custom_return_street]' value="<?php echo esc_attr( ( ! empty( $options['custom_return_street'] ) ? $options['custom_return_street'] : '' ) ); ?>">
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_return_zipcode',
			__( 'Zipcode.', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				?>
				<input id="gls_custom_return_zipcode" type='text' name='boostr_shipping_gls_settings[custom_return_zipcode]' value="<?php echo esc_attr( ( ! empty( $options['custom_return_zipcode'] ) ? $options['custom_return_zipcode'] : '' ) ); ?>">
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		add_settings_field(
			'custom_return_city',
			__( 'By', 'woocommerce-boostr-shipping' ),
			function() {
				$options = get_option( 'boostr_shipping_gls_settings' );
				?>
				<input id="gls_custom_return_city" type='text' name='boostr_shipping_gls_settings[custom_return_city]' value="<?php echo esc_attr( ( ! empty( $options['custom_return_city'] ) ? $options['custom_return_city'] : '' ) ); ?>">
				<?php
			},
			'boostr_shipping_gls_group',
			'boostr_shipping_gls'
		);

		register_setting( 'boostr_shipping_ftp_group', 'boostr_shipping_ftp_settings' );

		add_settings_section(
			'boostr_shipping_ftp',
			__( 'FTP settings', 'woocommerce-boostr-shipping' ),
			array( $this, 'ftp_settings_section_callback' ),
			'boostr_shipping_ftp_group'
		);

		add_settings_field(
			'export_ftp_address',
			__( 'FTP Address', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_address_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		add_settings_field(
			'export_ftp_port',
			__( 'FTP Port', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_port_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		add_settings_field(
			'export_ftp_username',
			__( 'FTP Username', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_username_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		add_settings_field(
			'export_ftp_password',
			__( 'FTP Password', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_password_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		add_settings_field(
			'export_ftp_folder',
			__( 'FTP Folder', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_folder_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		add_settings_field(
			'export_ftp_use_ssl',
			__( 'Use SSL', 'woocommerce-boostr-shipping' ),
			array( $this, 'export_ftp_use_ssl_render' ),
			'boostr_shipping_ftp_group',
			'boostr_shipping_ftp'
		);

		register_setting( 'boostr_shipping_google_group', 'boostr_shipping_google_settings' );

		add_settings_section(
			'boostr_shipping_google',
			__( 'Google Map settings', 'woocommerce-boostr-shipping' ),
			array( $this, 'google_settings_section_callback' ),
			'boostr_shipping_google_group'
		);

		add_settings_field(
			'google_map_api_key',
			__( 'Google Map API Key', 'woocommerce-boostr-shipping' ),
			array( $this, 'google_map_api_render' ),
			'boostr_shipping_google_group',
			'boostr_shipping_google'
		);

	}
	public function google_map_api_render() {
		$options = get_option( 'boostr_shipping_google_settings' );
		?>
		<input id="gls_contact_id" type='text' name='boostr_shipping_google_settings[google_map_api_key]' value="<?php echo esc_attr( ( ! empty( $options['google_map_api_key'] ) ) ? $options['google_map_api_key'] : '' ); ?>">
		<?php
	}

	public function google_settings_section_callback() {
		echo wp_kses_post( __( '</br>You can get a Google Maps API key from:  <a target="_blank" href="https://developers.google.com/maps/documentation/javascript/get-api-key">Google</a>', 'woocommerce-boostr-shipping' ) );
	}

	public function general_settings_section_callback() {
		$license_key = get_option( 'boostr_shipping_license_key' );
		$notice      = array();

		echo '<div class="wrap">';

		if ( isset( $_REQUEST['activate_license'], $_POST['_wpnonce'] ) && wp_verify_nonce( $_POST['_wpnonce'], 'boostr-shipping-manage-license' ) ) {
			if ( isset( $_REQUEST['boostr_shipping_terms_condition'] ) ) {
				$license_key = $_REQUEST['boostr_shipping_license_key'];
				// API query parameters
				$api_params = array(
					'slm_action'        => 'slm_activate',
					'secret_key'        => BOOSTR_SHIPPING_SECRET_KEY,
					'license_key'       => $license_key,
					'registered_domain' => $_SERVER['SERVER_NAME'],
					'item_reference'    => rawurlencode( BOOSTR_SHIPPING_ITEM_REFERENCE ),
					'product_id'        => '546',
					'prod_sku'          => 'B91GLS',
				);

				// Send query to the license manager server
				$query    = esc_url_raw( add_query_arg( $api_params, BOOSTR_SHIPPING_LICENSE_SERVER_URL ) );
				$response = wp_remote_get(
					$query,
					array(
						'timeout'   => 20,
						'sslverify' => false,
					)
				);

				// Check for error in the response
				if ( is_wp_error( $response ) ) {
					$notice['wp_error'] = __( 'Unexpected Error! The query returned with an error - License Key', 'woocommerce-boostr-shipping' );
				}

				// License data.
				$license_data = json_decode( wp_remote_retrieve_body( $response ) );

				if ( ! empty( $license_data ) ) {
					if ( $license_data->result == 'success' ) {//Success was returned for the license activation

						//Uncomment the followng line to see the message that returned from the license server
						$notice[] = __( 'The following message was returned from the server:', 'woocommerce-boostr-shipping' ) . '<strong>' . $license_data->message . '</strong>';

						//Save the license key in the options table
						update_option( 'boostr_shipping_license_key', $license_key );
						update_option( 'boostr_shipping_terms_condition', 1 );
					} else {
						//Show error to the user. Probably entered incorrect license key.
						//Uncomment the followng line to see the message that returned from the license server
						$notice[] = __( 'The following message was returned from the server:', 'woocommerce-boostr-shipping' ) . '<strong>' . $license_data->message . '</strong>';
					}
				} else {
					$notice[] = __( 'Something went wrong', 'woocommerce-boostr-shipping' );
				}
			} else {
				$notice[] = __( 'The following message was returned from the server: You must read and accept the terms and coditions below, before you can activate this plugin', 'woocommerce-boostr-shipping' );
			}
		}

		if ( isset( $_REQUEST['deactivate_license'], $_POST['_wpnonce'] ) && wp_verify_nonce( $_POST['_wpnonce'], 'boostr-shipping-manage-license' ) ) {
			$license_key = $_REQUEST['boostr_shipping_license_key'];

			// API query parameters
			$api_params = array(
				'slm_action'        => 'slm_deactivate',
				'secret_key'        => BOOSTR_SHIPPING_SECRET_KEY,
				'license_key'       => $license_key,
				'registered_domain' => $_SERVER['SERVER_NAME'],
				'item_reference'    => rawurlencode( BOOSTR_SHIPPING_ITEM_REFERENCE ),
				'product_id'        => '546',
				'prod_sku'          => 'B91GLS',
			);

			// Send query to the license manager server
			$query    = esc_url_raw( add_query_arg( $api_params, BOOSTR_SHIPPING_LICENSE_SERVER_URL ) );
			$response = wp_remote_get(
				$query,
				array(
					'timeout'   => 20,
					'sslverify' => false,
				)
			);

			// Check for error in the response
			if ( is_wp_error( $response ) ) {
				$notice[] = __( 'Unexpected Error! The query returned with an error - License Key', 'woocommerce-boostr-shipping' );
			}

			// License data.
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );
			if ( ! empty( $license_data ) ) {
				if ( $license_data->result == 'success' ) {//Success was returned for the license activation

					//Uncomment the followng line to see the message that returned from the license server
					$notice[] = __( 'The following message was returned from the server:', 'woocommerce-boostr-shipping' ) . '<strong>' . $license_data->message . '</strong>';

					//Remove the licensse key from the options table. It will need to be activated again.
					update_option( 'boostr_shipping_license_key', '' );
					update_option( 'boostr_shipping_terms_condition', '' );
				} else {
					//Show error to the user. Probably entered incorrect license key.
					update_option( 'boostr_shipping_license_key', '' );
					//Uncomment the followng line to see the message that returned from the license server
					$notice[] = __( 'The following message was returned from the server:', 'woocommerce-boostr-shipping' ) . '<strong>' . $license_data->message . '</strong>';
				}
			} else {
				$notice[] = __( 'Something went wrong', 'woocommerce-boostr-shipping' );
			}
		}

		foreach ( $notice as $value ) {
			echo '<div class="notice inline"><p>' . esc_html( $value ) . '</p></div>';
		}

		?>
		<?php
		if ( ! empty( get_option( 'boostr_shipping_license_key' ) ) ) {
			echo esc_html__( 'You currently have a active license key, running on your site', 'woocommerce-boostr-shipping' );
		} else {
			echo esc_html__( 'You currently have no active license key on this site, please activate the license key you got in the email, when purchasing this plugin.', 'woocommerce-boostr-shipping' );
		}
		?>

		<form action="" method="post">
		<?php wp_nonce_field( 'boostr-shipping-manage-license' ); ?>
			<table class="form-table">
				<tr>
					<th style="width:100px;">
						<label for="boostr_shipping_license_key"><?php echo esc_html__( 'License Key: ', 'woocommerce-boostr-shipping' ); ?></label>
					</th>
					<td ><input class="regular-text" type="text" id="boostr_shipping_license_key" name="boostr_shipping_license_key"  value="<?php echo esc_attr( ( ! empty( get_option( 'boostr_shipping_license_key' ) ) ) ? get_option( 'boostr_shipping_license_key' ) : '' ); ?>" ></td>
				</tr>
				<tr>
					<td></td>
					<td><input type="checkbox" name="boostr_shipping_terms_condition" id="boostr_shipping_terms_condition" <?php checked( get_option( 'boostr_shipping_terms_condition' ), 1 ); ?> value="1">
						<?php echo esc_html__( 'I accept the terms and conditions - ', 'woocommerce-boostr-shipping' ); ?>
						<span class="boostr_shipping_see_terms_and_condition"><?php echo esc_html__( 'Click Here', 'woocommerce-boostr-shipping' ); ?></span></td>
				</tr>
				<tr>
					<td></td>
					<td>
						<div class="boostr_shipping_terms_condition_text">
							<?php echo esc_html__( 'When using the GLS plugin all configuration is your responsibility to conduct correctly. All guides represent setup examples and is not guaranteeing that a specific setup can be achieved for your shop. Before using the plugin, we kindly ask you to check that you have a valid agreement with GLS and if you plan to use international GLS shipping that you have GLS agreements for all countries you plan to ship to.', 'woocommerce-boostr-shipping' ); ?>
							<ul>
								<li class="boostr_shipping_li_header"><?php echo esc_html__( 'Currently Supported GLS countries:', 'woocommerce-boostr-shipping' ); ?> </li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Denmark', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Austria', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Belgium', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Croatia', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Czech Republic', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Finland', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'France', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Germany', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Hungary', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Ireland', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Luxembourg', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Netherlands', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Poland', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Portugal', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Romania', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Slovakia', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Slovenia', 'woocommerce-boostr-shipping' ); ?></li>
								<li class="boostr_shipping_li_country"><?php echo esc_html__( 'Spain', 'woocommerce-boostr-shipping' ); ?></li>
							</ul>
						</div>
					</td>
				</tr>
			</table>

			<p class="submit">

				<?php
				if ( ! empty( get_option( 'boostr_shipping_license_key' ) ) ) {
					?>
					<input type="submit" name="deactivate_license" value="<?php echo esc_attr__( 'Deactivate', 'woocommerce-boostr-shipping' ); ?>" class="button-primary" />
					<?php

				} else {
					?>
					<input type="submit" name="activate_license" value="<?php echo esc_attr__( 'Activate', 'woocommerce-boostr-shipping' ); ?>" class="button-primary" />
					<?php
				}
				?>
			</p>
		</form>
		<?php
		echo '</div>';
	}

	public function auto_tracktrace_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );
		?>
		<select name='boostr_shipping_gls_settings[auto_tracktrace]'>
			<option value="No" <?php selected( $options['auto_tracktrace'], 'No' ); ?>><?php echo esc_html__( 'No', 'woocommerce-boostr-shipping' ); ?></option>
			<option value="Yes" <?php selected( $options['auto_tracktrace'], 'Yes' ); ?>><?php echo esc_html__( 'Yes', 'woocommerce-boostr-shipping' ); ?></option>
		</select>
		<?php
	}

	public function custom_tracktrace_email_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );

		?>
		<textarea cols='60' rows='8' name='boostr_shipping_gls_settings[custom_tracktrace_email]'>
		<?php
		if ( ! empty( $options['custom_tracktrace_email'] ) ) {
			echo esc_html( trim( $options['custom_tracktrace_email'] ) );
		} else {
			echo esc_html( __( 'Your package will be mailed shortly, you can follow your package at the link below or by entering the track and trace ID on https://gls-group.eu', 'woocommerce-boostr-shipping' ) );
		}
		?>
			</textarea>
		<?php
	}

	public function gls_customer_id_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );
		?>
		<input id="gls_customer_id" type='text' name='boostr_shipping_gls_settings[gls_customer_id]' value="<?php echo esc_attr( ( ! empty( $options['gls_customer_id'] ) ) ? $options['gls_customer_id'] : '' ); ?>">
		<?php
	}

	public function gls_contact_id_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );
		?>
		<input id="gls_contact_id" type='text' name='boostr_shipping_gls_settings[gls_contact_id]' value="<?php echo esc_attr( ( ! empty( $options['gls_contact_id'] ) ) ? $options['gls_contact_id'] : '' ); ?>">
		<?php
	}

	public function gls_username_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );
		?>
		<input id="gls_username" type='text' name='boostr_shipping_gls_settings[gls_username]' value="<?php echo esc_attr( ( ! empty( $options['gls_username'] ) ) ? $options['gls_username'] : '' ); ?>">
		<?php
	}

	public function gls_password_render() {
		$options = get_option( 'boostr_shipping_gls_settings' );
		?>
		<input id="gls_password" type='password' name='boostr_shipping_gls_settings[gls_password]' value="<?php echo esc_attr( ( ! empty( $options['gls_password'] ) ) ? $options['gls_password'] : '' ); ?>">
		<?php
	}

	public function export_ftp_address_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		?>
		<input type='text' name='boostr_shipping_ftp_settings[export_ftp_address]' value="<?php echo esc_attr( ( ! empty( $options['export_ftp_address'] ) ) ? $options['export_ftp_address'] : '' ); ?>">
		<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'The FTP Host Address of the server where the files are saved - for example: test2.example.dk', 'woocommerce-boostr-shipping' ); ?></div>
		<?php
	}

	public function export_ftp_port_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		?>
		<input type='text' name='boostr_shipping_ftp_settings[export_ftp_port]' value="<?php echo esc_attr( ( ! empty( $options['export_ftp_port'] ) ) ? $options['export_ftp_port'] : '' ); ?>">
		<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'The FTP Port number of the server where the files are saved - for example:  21', 'woocommerce-boostr-shipping' ); ?></div>
		<?php
	}

	public function export_ftp_username_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		?>
		<input type='text' name='boostr_shipping_ftp_settings[export_ftp_username]' value="<?php echo esc_attr( ( ! empty( $options['export_ftp_username'] ) ) ? $options['export_ftp_username'] : '' ); ?>">
		<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'The FTP username used on the server to login', 'woocommerce-boostr-shipping' ); ?></div>
		<?php
	}

	public function export_ftp_password_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		?>
		<input type='password' name='boostr_shipping_ftp_settings[export_ftp_password]' value="<?php echo esc_attr( ( ! empty( $options['export_ftp_password'] ) ) ? $options['export_ftp_password'] : '' ); ?>">
		<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'The FTP password used on the server to login', 'woocommerce-boostr-shipping' ); ?></div>
		<?php
	}

	public function export_ftp_folder_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		?>
			<input type='text' name='boostr_shipping_ftp_settings[export_ftp_folder]' value="<?php echo esc_attr( ( ! empty( $options['export_ftp_folder'] ) ) ? $options['export_ftp_folder'] : '' ); ?>">
			<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'The name of the folder the files are saved on - for example:  "uploads/labels/gls', 'woocommerce-boostr-shipping' ); ?></div>
			<div class="boostr-shipping-helper-text"><?php echo esc_html__( 'If you wish it to store it on the root of the server, leave this field empty', 'woocommerce-boostr-shipping' ); ?></div>
		<?php
	}

	public function export_ftp_use_ssl_render() {
		$options = get_option( 'boostr_shipping_ftp_settings' );
		if ( empty( $options['export_ftp_use_ssl'] ) ) {
			$options['export_ftp_use_ssl'] = 'Yes';
		}
		?>
		<select name='boostr_shipping_ftp_settings[export_ftp_use_ssl]'>
			<option value="No" <?php selected( $options['export_ftp_use_ssl'], 'No' ); ?>><?php echo esc_html__( 'No', 'woocommerce-boostr-shipping' ); ?></option>
			<option value="Yes" <?php selected( $options['export_ftp_use_ssl'], 'Yes' ); ?>><?php echo esc_html__( 'Yes', 'woocommerce-boostr-shipping' ); ?></option>
		</select>
		<?php
	}

	public function ftp_settings_section_callback() {
		echo esc_html__( 'Please make sure that you have writing permission, on your FTP server before you attempt to setup the FTP export', 'woocommerce-boostr-shipping' );
		echo '<br/>';
		echo esc_html__( 'You can get your FTP information, from the company hosting your website - if in doubt!', 'woocommerce-boostr-shipping' );
	}

	public function gls_settings_section_callback() {
		$url = '<a href="' . plugins_url( 'boostr-shipping' ) . '/assets/images/How-to-find-gls-information.png">GLS information</a>';
		printf( esc_html__( 'Click at %s to find your information  ', 'woocommerce-boostr-shipping' ), wp_kses_post( $url ) );
	}

	public function options_page() {

		?>
			<div class="wrap">
				<h2><?php echo esc_html__( 'Plugins.dk GLS Settings', 'woocommerce-boostr-shipping' ); ?></h2>
				<?php settings_errors(); ?>

				<?php
					$active_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 'boostr_shipping_general';
				?>

				<h2 class="nav-tab-wrapper">
					<a href="?page=boostr-shipping&tab=boostr_shipping_general" class="nav-tab <?php echo $active_tab == 'boostr_shipping_general' ? 'nav-tab-active' : ''; ?>">General</a>
					<a href="?page=boostr-shipping&tab=boostr_shipping_gls" class="nav-tab <?php echo $active_tab == 'boostr_shipping_gls' ? 'nav-tab-active' : ''; ?>">GLS</a>
					<a href="?page=boostr-shipping&tab=boostr_shipping_ftp" class="nav-tab <?php echo $active_tab == 'boostr_shipping_ftp' ? 'nav-tab-active' : ''; ?>">FTP</a>
					<a href="?page=boostr-shipping&tab=boostr_shipping_google" class="nav-tab <?php echo $active_tab == 'boostr_shipping_google' ? 'nav-tab-active' : ''; ?>">Map</a>
				</h2>

				<?php
				if ( $active_tab == 'boostr_shipping_general' ) {
					settings_fields( 'boostr_shipping_general_group' );
					do_settings_sections( 'boostr_shipping_general_group' );
				} else {
					?>
						<form method="post" action="options.php">
							<?php
							if ( $active_tab == 'boostr_shipping_gls' ) {
								settings_fields( 'boostr_shipping_gls_group' );
								do_settings_sections( 'boostr_shipping_gls_group' );
							} elseif ( $active_tab == 'boostr_shipping_ftp' ) {
								settings_fields( 'boostr_shipping_ftp_group' );
								do_settings_sections( 'boostr_shipping_ftp_group' );
							} elseif ( $active_tab == 'boostr_shipping_google' ) {
								settings_fields( 'boostr_shipping_google_group' );
								do_settings_sections( 'boostr_shipping_google_group' );
							}
							submit_button();
							?>
						</form>
					<?php
				}
				?>

			</div>
		<script>
			jQuery( document ).ready(function() {

				jQuery('.boostr_shipping_see_terms_and_condition').on('click', function()
				{
					jQuery('.boostr_shipping_terms_condition_text').slideToggle();
				})

			});

		</script>
		<?php
	}

	public function plugin_action_links( $links ) {
		$plugin_links = array(
			'settings' => '<a href="' . admin_url( 'admin.php?page=boostr-shipping' ) . '" aria-label="' . esc_attr__( 'View Plugins.dk GLS settings', 'woocommerce-boostr-shipping' ) . '">' . esc_html__( 'Settings', 'woocommerce-boostr-shipping' ) . '</a>',
		);

		return array_merge( $plugin_links, $links );
	}
}
