<?php

defined( 'ABSPATH' ) || exit;

require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/boostr_shipping_main.php';

class Boostr_Shipping_GLS_Business extends Boostr_Shipping_Main {
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'boostr_shipping_gls_business';
		$this->instance_id        = absint( $instance_id );
		$this->method_title       = __( 'GLS business', 'woocommerce-boostr-shipping' );
		$this->method_description = __( 'Adds the option to ship with the GLS business to the checkout', 'woocommerce-boostr-shipping' );
		$this->supports           = array(
			'shipping-zones',
			'instance-settings',
		);

		$this->init();
	}

	/* add the diffrent actions */
	public function addActions() {
		//adds the shipping method to the WooCommerce
		add_filter( 'woocommerce_shipping_methods', array( $this, 'register_shipping_method' ) );
		add_action( 'woocommerce_after_shipping_rate', array( $this, 'boostr_shipping_gls_business_show_below_shipping' ) );
		add_action( 'woocommerce_checkout_process', array( $this, 'boostr_shipping_gls_business_field_process' ) );
	}

	public function addFilters() {

	}

	public function boostr_shipping_gls_business_field_process() {
		global $woocommerce;
		$choosen_shipping_method1 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods[0] );
		$choosen_shipping_method2 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods );
		if ( ( isset( $_POST['ship_to_different_address'] ) && ( ! isset( $_POST['shipping_company'] ) || $_POST['billing_company'] == '' ) ) && ( $choosen_shipping_method1 == 'boostr_shipping_gls_business' || $choosen_shipping_method2 == 'boostr_shipping_gls_business' ) ) {
				wc_add_notice( __( 'Please fill out the Shipping company', 'woocommerce-boostr-shipping' ), 'error' );
		}

		if ( ( ! isset( $_POST['ship_to_different_address'] ) && ( ! isset( $_POST['shipping_company'] ) || $_POST['billing_company'] == '' ) ) && ( $choosen_shipping_method1 == 'boostr_shipping_gls_business' || $choosen_shipping_method2 == 'boostr_shipping_gls_business' ) ) {
				wc_add_notice( __( 'Please fill out the billing company', 'woocommerce-boostr-shipping' ), 'error' );
		}
	}

	public function boostr_shipping_gls_business_show_below_shipping( $rate ) {
		global $woocommerce;

		$choosen_shipping_method1 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods[0] );
		$choosen_shipping_method2 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods );
		if ( $choosen_shipping_method1 == 'boostr_shipping_gls_business' || $choosen_shipping_method2 == 'boostr_shipping_gls_business' ) {
			if ( $rate->method_id == 'boostr_shipping_gls_business' ) {
				echo '<div class="gls_shipping_method_text">' . esc_html__( 'The company name is required.', 'woocommerce-boostr-shipping' ) . '</div>';
			}
		}
	}

	/* Register the shipping method in WooCommerce*/
	public function register_shipping_method( $methods ) {
		$methods['boostr_shipping_gls_business'] = 'Boostr_Shipping_GLS_Business';
		return $methods;
	}
}

$boostr_GLS_Business = new Boostr_Shipping_GLS_Business();
$boostr_GLS_Business->mainAddAction();
$boostr_GLS_Business->addActions();
$boostr_GLS_Business->addFilters();
