<?php

defined( 'ABSPATH' ) || exit;

require_once BOOSTR_SHIPPING_PLUGIN_DIR . '/includes/boostr_shipping_main.php';
class Boostr_Shipping_GLS_Pickup extends Boostr_Shipping_Main {
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'boostr_shipping_gls_pickup';
		$this->instance_id        = absint( $instance_id );
		$this->method_title       = __( 'GLS pickup point', 'woocommerce-boostr-shipping' );
		$this->method_description = __( 'Adds the option to ship with the GLS pickup point to the checkout', 'woocommerce-boostr-shipping' );
		$this->supports           = array(
			'shipping-zones',
			'instance-settings',
		);

		$this->init();
	}

	/* add the diffrent actions */
	public function addActions() {
		//ajax for the pakkeshop list
		add_action( 'wp_ajax_boostr_shipping_get_shop_list', array( $this, 'get_shop_list_callback' ) );
		add_action( 'wp_ajax_nopriv_boostr_shipping_get_shop_list', array( $this, 'get_shop_list_callback' ) );

		//adds the shipping method to the WooCommerce
		add_filter( 'woocommerce_shipping_methods', array( $this, 'register_shipping_method' ) );
		//runs the code after the choosen shipping method
		add_action( 'woocommerce_after_shipping_rate', array( $this, 'check_choosen_shipping_method' ) );

		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'boostr_shipping_checkout_update_order_meta_method_boostr_shipping' ) );
		//Makes sure you cant buy without selecting a packet shop
		add_action( 'woocommerce_checkout_process', array( $this, 'boostr_shipping_checkout_process_shipping' ) );
		add_action( 'wp_footer', array( $this, 'render_pickup_modal' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 99 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
	}

	public function addFilters() {

	}

	public function enqueue_scripts() {
		if ( is_checkout() ) {
			$iGoogle_map_api = get_option( 'boostr_shipping_google_settings' )['google_map_api_key'];
			$aParams         = array(
				'ajax_url'              => admin_url( 'admin-ajax.php' ),
				'plugin_root'           => BOOSTR_SHIPPING_PLUGIN_URL,
				'selected_shop_header'  => __( 'Currently choosen pickup point:', 'woocommerce-boostr-shipping' ),
				'error_no_shops_found'  => __( 'No pickup points found for the current choosen Zipcode:', 'woocommerce-boostr-shipping' ),
				'error_no_cords_found'  => __( '* Couldnt mark this pickup point on the map', 'woocommerce-boostr-shipping' ),
				'sMonday'               => __( 'Monday', 'woocommerce-boostr-shipping' ),
				'sTuesday'              => __( 'Tuesday', 'woocommerce-boostr-shipping' ),
				'sWednesday'            => __( 'Wednesday', 'woocommerce-boostr-shipping' ),
				'sThursday'             => __( 'Thursday', 'woocommerce-boostr-shipping' ),
				'sFriday'               => __( 'Friday', 'woocommerce-boostr-shipping' ),
				'sSaturday'             => __( 'Saturday', 'woocommerce-boostr-shipping' ),
				'sSunday'               => __( 'Sunday', 'woocommerce-boostr-shipping' ),
				'sWrongLicensCode'      => __( 'Sorry, the license key used for this shipping method didnt work', 'woocommerce-boostr-shipping' ),
				'sPacketshop'           => __( 'Packetshop:', 'woocommerce-boostr-shipping' ),
				'sNoPacketshopClose'    => __( 'Couldnt find any pickup point close to the choosen zipcode', 'woocommerce-boostr-shipping' ),
				'sSorryDisabledCountry' => __( 'Sorry, this shipping with this method for your choosen country is currently disabled', 'woocommerce-boostr-shipping' ),
				'sEnterZipcode'         => __( 'You must enter a zipcode, to see the nearest pickup points - please try again', 'woocommerce-boostr-shipping' ),
			);

			wp_enqueue_script( 'gmaps', 'https://maps.googleapis.com/maps/api/js?key=' . $iGoogle_map_api, array( 'jquery' ), BOOSTR_SHIPPING_VERSION, false );
			wp_enqueue_script( 'boostr_shipping_script', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/js/frontend/boostr-shipping.js', array( 'jquery', 'gmaps' ), BOOSTR_SHIPPING_VERSION );
			wp_localize_script( 'boostr_shipping_script', 'BoostrShippingParams', $aParams );

			if ( ! $this->active_theme_is_conflicting() ) {
				wp_enqueue_script( 'modal-script', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/js/frontend/boostr-shipping-modal.js', array( 'jquery' ), BOOSTR_SHIPPING_VERSION, true );
			}
		}
	}

	public function enqueue_styles() {
		if ( is_checkout() ) {
			wp_enqueue_style( 'woocommerce-boostr_shipping', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/css/frontend/boostr-shipping.css', array(), BOOSTR_SHIPPING_VERSION );
			wp_enqueue_style( 'boostr-shipping-modal-css', BOOSTR_SHIPPING_PLUGIN_URL . '/assets/css/frontend/boostr-shipping-modal.css', array(), BOOSTR_SHIPPING_VERSION );
		}
	}

	public function active_theme_is_conflicting() {
		//Conflicting known themes
		$bConflictingTheme  = 0;
		$oConflictingThemes = array(
			'0' => 'avada',
			'1' => 'avada_child',

		);

		foreach ( $oConflictingThemes as $theme ) {
			if ( strtolower( wp_get_theme() ) == strtolower( $theme ) ) {
				$bConflictingTheme = 1;

			}
		}

		return $bConflictingTheme;
	}

	public function get_shop_list_callback() {
		global $woocommerce;
		$response['status'] = true;

		if ( ! isset( $_POST['zipcode'] ) || strlen( $_POST['zipcode'] ) !== 4 || ! ctype_digit( $_POST['zipcode'] ) ) {
			$response['status'] = false;
			$response['error']  = __( 'You must enter a zipcode, to see the nearest pickup points - please try again', 'woocommerce-boostr-shipping' );
			print wp_json_encode( $response );
			wp_die();
		}

		$data = array(
			'sFunction'   => 'searchshops',
			'sCountry'    => $woocommerce->customer->get_shipping_country(),
			'iZipcode'    => $_POST['zipcode'],
			'sLicenseKey' => get_option( 'boostr_shipping_license_key' ),
		);

		$result = $this->get_shop_list( $data );

		if ( is_wp_error( $result ) || strpos( $result, 'shoplist_json' ) === false ) {
			// results not as expected. create an error
			$response['return_error'] = __( 'Something went wrong. Please try again later', 'woocommerce-boostr-shipping' );
				print wp_json_encode( $response );
		} else {
			if ( ! empty( $result ) ) {
				echo $result;
			} else {
				$response['error'] = __( 'You must enter a zipcode, to see the nearest pickup points - please try again', 'woocommerce-boostr-shipping' );
				print wp_json_encode( $response );
			}
		}

		wp_die();
	}

	public function get_shop_list( $data ) {
		$args = array_map( 'rawurlencode', $data );

		$url = add_query_arg(
			$args,
			BOOSTR_SHIPPING_LICENSE_SERVER_URL . '/wp-json/boostrshipping/v2/api'
		);

		$res = wp_safe_remote_get( $url );

		if ( is_wp_error( $res ) ) {
			return $res;
		}

		return wp_remote_retrieve_body( $res );
	}

	//Checks if a packetshop is selected, and its one of the valid shipping methods
	public function boostr_shipping_checkout_process_shipping() {
		global $woocommerce;
		$choosen_shipping_method1 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods[0] );
		$choosen_shipping_method2 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods );
		if ( $choosen_shipping_method1 == 'boostr_shipping_gls_pickup' || $choosen_shipping_method2 == 'boostr_shipping_gls_pickup' ) {
			if ( ! isset( $_POST['boostr_shipping'] ) || empty( $_POST['boostr_shipping'] ) ) {
				wc_add_notice( __( 'Please select a pickup point before placing your order.', 'woocommerce-boostr-shipping' ), 'error' );
			}
		}
	}

	//Updates the order with the shipping information of the choosen packetshop
	public function boostr_shipping_checkout_update_order_meta_method_boostr_shipping( $order_id ) {
		global $woocommerce;
		$choosen_shipping_method1 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods[0] );
		$choosen_shipping_method2 = preg_replace( '/\d/', '', $woocommerce->session->chosen_shipping_methods );
		if ( $choosen_shipping_method1 == 'boostr_shipping_gls_pickup' || $choosen_shipping_method2 == 'boostr_shipping_gls_pickup' ) {
			//checks if the hidden radio group is set, and uses the information to update the order
			if ( isset( $_POST['boostr_shipping'] ) && ! empty( $_POST['boostr_shipping'] ) ) {
				$first_name = ( isset( $_POST['ship_to_different_address'] ) ) ? $_POST['shipping_first_name'] : $_POST['billing_first_name'];
				$last_name  = ( isset( $_POST['ship_to_different_address'] ) ) ? $_POST['shipping_last_name'] : $_POST['billing_last_name'];

				update_post_meta( $order_id, '_shipping_first_name', $first_name );
				update_post_meta( $order_id, '_shipping_last_name', $last_name );

				$shopnumber = $_POST['boostr_shipping'];
				update_post_meta( $order_id, '_shipping_company', $_POST['shop_name'][ $shopnumber ] );
				update_post_meta( $order_id, '_shipping_address_1', $_POST['shop_address'][ $shopnumber ] );
				update_post_meta( $order_id, '_shipping_address_2', 'Pakkeshop: ' . $shopnumber );
				update_post_meta( $order_id, '_shipping_city', $_POST['shop_city'][ $shopnumber ] );
				update_post_meta( $order_id, '_shipping_postcode', $_POST['shop_zip'][ $shopnumber ] );
				add_post_meta( $order_id, 'shop_id', $shopnumber, true );
			}
		}
	}

	/* Register the shipping method in WooCommerce*/
	public function register_shipping_method( $methods ) {
		$methods['boostr_shipping_gls_pickup'] = 'Boostr_Shipping_GLS_Pickup';
		return $methods;
	}

	/* Checks the choosen shipping method, and adds */
	public function check_choosen_shipping_method( $rate ) {
		$chosen_methods  = WC()->session->get( 'chosen_shipping_methods' );
		$chosen_shipping = preg_replace( '/[0-9]+/', '', $chosen_methods[0] );

		if ( $chosen_shipping === 'boostr_shipping_gls_pickup' ) {
			if ( $rate->method_id == 'boostr_shipping_gls_pickup' ) {
				$this->HTML_zipAndFind( 'gls' );
			}
		}
	}

	public function render_pickup_modal() {
		if ( is_checkout() ) {
			?>
			<div class="boostr-shipping-modal fade" id="boostr-shipping-modal" tabindex="-1" role="dialog"
				aria-labelledby="myModalLabel">
				<div class="boostr-shipping-modal-dialog" role="document">
					<div class="boostr-shipping-modal-content boostr-shipping-list-wrap">
						<div class="boostr-shipping-modal-header">
							<button type="button" class="close" data-dismiss="modal" aria-label="Close">
								<span aria-hidden="true">&times;</span></button>
							<div class="boostr-shipping-modal-title"
								id="modal-title"><?php echo esc_html__( 'Choose a pickup point', 'woocommerce-boostr-shipping' ); ?></div>
						</div>
						<div id="inner-modal-buddy" class="boostr-shipping-modal-body">
						</div>
						<div class="boostr-shipping-modal-footer">
							<button type="button"
									class="button text-show-map js-show_map"><?php echo esc_html__( 'Show Map', 'woocommerce-boostr-shipping' ); ?></button>
							<button type="button"
									class="button text-hide-map js-show_map"><?php echo esc_html__( 'Hide Map', 'woocommerce-boostr-shipping' ); ?></button>
							<button type="button"
									class="button text-hide-list js-show_map"><?php echo esc_html__( 'Hide list', 'woocommerce-boostr-shipping' ); ?></button>
							<button type="button" class="button"
									data-dismiss="modal"><?php echo esc_html__( 'Choose pickup point', 'woocommerce-boostr-shipping' ); ?></button>
						</div>
					</div>
					<div id="map_wrapper" class="boostr-shipping-modal-content map_wrapper">
						<div class="boostr-shipping-modal-header">
							<button type="button" class="close" data-dismiss="modal" aria-label="Close">
								<span aria-hidden="true">&times;</span></button>
							<div class="boostr-shipping-modal-title"
								id="boostr-shipping-modal-title"><?php echo esc_html__( 'Choose a pickup point', 'woocommerce-boostr-shipping' ); ?></div>
						</div>
						<div class="boostr-shipping-modal-body map_content_wrapper">
						</div>
						<div class="boostr-shipping-modal-footer">
							<button type="button"
									class="button text-hide-map js-show_map"><?php echo esc_html__( 'Hide Map', 'woocommerce-boostr-shipping' ); ?></button>
							<button type="button"
									class="button text-show-list js-show_map"><?php echo esc_html__( 'Show list', 'woocommerce-boostr-shipping' ); ?></button>
							<button type="button" id="boostr-choose-shop-btn" class="button "
									data-dismiss="modal"><?php echo esc_html__( 'Choose pickup point', 'woocommerce-boostr-shipping' ); ?></button>
						</div>
					</div>
				</div>
			</div>
			<?php
		}
	}
}

$boostr_GLS_Pickup = new Boostr_Shipping_GLS_Pickup();
$boostr_GLS_Pickup->mainAddAction();
$boostr_GLS_Pickup->addActions();
$boostr_GLS_Pickup->addFilters();
